#include "escfp23x.h" // explode
#include "gamefunc.h"
#include "aflight.h"
#include "const.h"

const float REGRATE = 0.25;

// the actual game
int FlightModel::start(Shared *s)
{
  shared = s;

  subgamestat = 0;
  fuelRate = 0.0;
  regRate = 0.0;
  currentSpeed = speed = shared->zone.speed;
  perilBonus = 0;
  fuelEff = 0.125;

  switch(difficulty)
  {
    case D_EASY:
      manuverPenalty = 0.10;
      break;
    case D_MEDIUM:
      manuverPenalty = 0.15;
      break;
    case D_HARD:
      manuverPenalty = 0.16;
      break;
    case D_RIDICULOUS:
      manuverPenalty = 0.175;
      break;
  }

  sm = new SpriteManager(150);
  sm->initialize(shared);
  sm->wallReset();

  shared->zone.prob[ST_METEOR] += shared->zone.meteorMod;
  if (shared->zone.prob[ST_METEOR] < 0)
    shared->zone.prob[ST_METEOR] = 0;

  shared->zone.prob[ST_POWER] += shared->zone.powerMod;
  if (shared->zone.prob[ST_POWER] < 0)
    shared->zone.prob[ST_POWER] = 0;

  shared->player.placeMessage(
    shared->zone.maxleft+((shared->zone.maxright-shared->zone.maxleft)/2)-6,
    shared->zone.maxtop+((shared->zone.maxbottom-shared->zone.maxtop)/2));

  shared->player.active().soil(ALOT);
  shared->player.select().soil(ALOT);
  shared->player.updateInfo(ALOT);

  while(subgamestat == 0)
    go();

  return subgamestat;
}

void FlightModel::go(void)
{
  distance = shared->zone.length;
  oldbc = shared->zone.bcolor;
  opening = 9;
  subgamestat = 0;

  switch(shared->zone.scrollx+(shared->zone.scrolly*2))
  {
    case 1: y = 13; x = 22; break;
    case -1: y = 13; x = 4; break;
    case 2: x = 13; y = 22; break;
    case -2: x = 13; y = 4; break;
  }

  dx = 0;
  dy = 0;

  sm->reset();

  shipNose = sm->findFreeSprite();
  shipBody = sm->findFreeSprite();
  shipFire = sm->findFreeSprite();
  if (shipNose && shipBody && shipFire)
    sm->ship3(shipNose, shipBody, shipFire, x, y);
  else
  {
    cputs("No Ship!");
    getch();
  }

  itemOnOff(&(shared->player.items[T_SHIELD]));
  if (invinc > 2)
  {
    shipNose->reColor(YELLOW);
    shipBody->reColor(YELLOW);
  }

  drawWalls();

  //creates stars
  sm->spawn(x, y, dx, dy, distance, 1);

  // yes, a blatant hack.
  if (warpTime > -1)
  {
    int temp = warpTime;
    warpTime = -1;
    warpOn(temp);
  }

  frameControl.set(currentSpeed);
  frameControl.sync();

  while(subgamestat == 0)
    run();

  if (subgamestat > 1)
    for (int a = 1000;a > 0;a -= 50)
    {
      speaker.play(a);
      delay(50);
    }
  speaker.stop();

  if (subgamestat == 4)
    subgamestat = 0;
  else if (subgamestat == 1)
    explode(x,y);

  delay(100);
  while(getkey())
    ;

  shared->player.items[T_WARP].on = 0;

  itemOnOff(&(shared->player.items[T_WARP]));

  shared->player.items[T_WARP].soil(ALOT);

}

void FlightModel::drawWalls(void)
{
  int a, b, c;
  // walls in constrained areas
  textbox(1, 1, 25, 25);
  cset(RED, BROWN);
  for (c = 2;c < 12;c+=2)
  {
    for (a = c;a < c+2;a++)
    {
      for (b = 2;b < 25;b++)
      {
        if (shared->zone.scrollx == 0)
          gotoxy(a, b);
        else
          gotoxy(b, a);
        switch (c)
        {
          case 2: putch(32); break;
          case 4: putch(176); break;
          case 6: putch(177); break;
          case 8: putch(178); break;
          case 10: putch(219); break;
        }
        if (shared->zone.scrollx == 0)
          gotoxy(a+13, b);
        else
          gotoxy(b, a+13);
        switch (c)
        {
          case 2: putch(32); break;
          case 4: putch(176); break;
          case 6: putch(177); break;
          case 8: putch(178); break;
          case 10: putch(219); break;
        }
      }
    }
  }

  blankBackground();

  // end background
}

void FlightModel::run(void)
{
  // game loop
  dx = 0;
  dy = 0;

  if (perilBonus == 0 &&
      shared->player.items[T_HITS].power < shared->player.items[T_HITS].pUse)
  {
    shared->zone.prob[ST_POWER]++;
    perilBonus = 1;
  }
  if (perilBonus == 1 &&
      shared->player.items[T_HITS].power >= shared->player.items[T_HITS].pUse)
  {
    shared->zone.prob[ST_POWER]--;
    perilBonus = 0;
  }

  consume();

  shipControl();

  // needs to be after control since turning Xshields off sets invinc = 2
  if (invincTime > 0)
  {
    invincTime--;
    invinc = 4;
    shared->player.invincBar(ALITTLE, invincTime);
  }

  int effects = 0;
  int bob = sm->playerCollide(x, y, dx, dy, invinc, effects);
  if (bob)
    subgamestat = bob;

  while (effects > 0)
  {
    int a, b;
    char blah[15] = {ASC_TRIANGLE_E, '\0'};
    shared->player.chooseRandomBonus(a, b);
    strcpy(blah+1, shared->player.makeRandomName(a, b));
    blah[13] = '\0';
    shared->player.setMessage(blah, 10);
    randomEffect(a, b);
    effects--;
  }
  sm->objectCollide(distance);

  shipNose->dx = dx;
  shipNose->dy = dy;
  shipBody->dx = dx;
  shipBody->dy = dy;
  shipFire->dx = dx;
  shipFire->dy = dy;

  if (shared->zone.special == 1)
    sm->spawn(x, y, random(3)-1, random(3)-1, distance);
  else
    sm->spawn(x, y, shared->zone.scrollx, shared->zone.scrolly, distance);

  doBoss();

  if (invinc > 0)
  {
    if (--invinc == 1)
    {
      shipNose->reColor(LIGHTGRAY);
      shipBody->reColor(LIGHTGRAY);
      itemOnOff(&(shared->player.items[T_SHIELD]));
    }
  }

  draw();

  // walls always move forward
  sm->wallForward();

  if (warpTime < 1 && shared->zone.deltaSpeed != 0)
  {
    shared->zone.speed += shared->zone.deltaSpeed;
    shared->zone.deltaSpeed = 0;

    if (shared->zone.speed < 5)
      shared->zone.speed = 5;

    speed = shared->zone.speed;
  }

  if (currentSpeed != speed)
  {
    bob = (speed - currentSpeed) * 0.1;
    if (bob == 0)
      bob = speed - currentSpeed;
    currentSpeed += bob;
    frameControl.set(currentSpeed);
  }

  frameControl.force();

  if (warpTime > 0)
  {
    warpSound += wsInc;
    if (warpSound < 200)
      wsInc = 2;
    if (warpSound > 300)
      wsInc = -1;
    speaker.play(warpSound);
  }
  else
    speaker.stop();

  // has to be last so sound works.
  if (warpTime < 0)
    ;
  else if (warpTime > 0)
  {
    warpTime--;
    invincTime = 40;
    shared->player.invincBar(ALOT, invincTime);
  }
  else // if warpTime == 0
  {
    speed = shared->zone.speed;
    shared->zone.bcolor = oldbc;
    blankBackground();
    warpTime--;
  }
}

void FlightModel::consume(void)
{
  int bob, thres;

  regRate += REGRATE;

  for (int a = 0;a < NUMTRINKETS;a++)
  {
    switch(shared->player.items[a].iAm)
    {
      // always
      case T_LASER:
        bob = shared->player.items[T_LASER].power/50;
        if (bob > 0 && shared->player.items[T_LASER].power % 50 < 49)
          shared->player.items[T_LASER].soil(ALOT);
        thres = 1;
        while (bob >= thres && shared->player.items[T_LASER].power % 50 < 49)
        {
          shared->player.items[T_LASER].power++;
          thres++;
        }
        break;
    }  

    // on effects
    if (shared->player.items[a].on)
    {
      switch(a)
      {
        case T_SHIELD:
          invinc = 2;
          break;
        case T_PHASE:
          invinc = 4;
          break;
        case T_WARP:
          warpOn(1);
          break;
	case T_RANDOM:
          if (shared->player.items[a].power >= shared->player.items[a].pUse
	   && shared->player.experience < MAXEXP-10)
	  {
	    speaker.play(500);
	    shared->player.experience = MAXEXP-2;
            shared->player.items[a].usePower();
	  }
      }
    }

    // consume power if on
    if (shared->player.items[T_WARP].on)
    {
      if (a == T_WARP)
      {
	shared->player.items[a].usePower();
	if (shared->player.items[a].power < shared->player.items[a].pUse)
	{
	  shared->player.items[a].on = 0;
	  shared->player.items[a].soil(ALOT);
	  itemOnOff(&(shared->player.items[a]));
	}
      }
    }
    else if (shared->player.items[a].on && warpTime < 1)
    {
      switch(a)
      {
        case T_FUEL:
          if ((fuelRate += fuelEff) >= 1.0)
          {
            shared->player.items[T_FUEL].usePower();
            fuelRate -= 1.0;
          }
          break;
	case T_RANDOM: break;
        default:
            shared->player.items[a].usePower();
            if (shared->player.items[a].power < shared->player.items[a].pUse)
            {
              shared->player.items[a].on = 0;
              shared->player.items[a].soil(ALOT);
              itemOnOff(&(shared->player.items[a]));
            }
          break;
      }
    }
  }

  if (regRate >= 1.0)
    regRate -= 1.0;
  if (warpTime < 1)
    shared->player.getPower();

}

void FlightModel::shipControl(void)
{
  int t;
  if (opening == 0)
  {
    switch (getkey())
    {
      case UP:
        if (y > shared->zone.maxtop
         && y+shared->zone.scrolly*2 > shared->zone.maxtop)
          dy = -1;
        break;
      case DOWN:
        if (y < shared->zone.maxbottom
         && y+shared->zone.scrolly*2 < shared->zone.maxbottom)
          dy = 1;
        break;
      case RIGHT:
        if (x < shared->zone.maxright
         && x+shared->zone.scrollx*2 < shared->zone.maxright)
          dx = 1;
        break;
      case LEFT:
        if (x > shared->zone.maxleft
         && x+shared->zone.scrollx*2 > shared->zone.maxleft)
          dx = -1;
        break;
      case SPACE:
        speaker.play(500);
        useItem();
        break;
      case 'v':
      case 'V':
        speaker.playFor(700, 50);
        shared->player.IBSelect(&(shared->player.itemA), CU_FLIGHT);
        break;
      case 'b':
      case 'B':
        if (shared->player.playMode == PM_NEW)
        {
          speaker.playFor(700, 50);
          shared->player.IBSelect(&(shared->player.itemB), CU_FLIGHT);
        }
        else
          useItem(&(shared->player.items[T_RANDOM]));
        break;
      case 'n':
      case 'N':
        if (shared->player.experience >= MAXEXP)
        {
          speaker.play(500);
          shared->player.experience = 0;
          shared->player.expBar(ALOT);
	  shared->player.clearMessage(1);
          randomEffect(shared->player.take, shared->player.subTake);
          shared->player.take = T_NONE;
        }
        else
          speaker.play(50);
        break;
      case ESCAPE:
        pause(subgamestat);
        shared->player.updateInfo(ALOT);
        break;
     #ifdef CHEAT
      case 'S': distance = 0; break;
      case 'u':
        shared->player.plus();
        break;
      case 'U':  // uber cheat
        {for (int a = 1;a < NUMTRINKETS;a++)
          shared->player.items[a].power = 1600;}
        break;
      case 'z': // zero fuel to check out-of-fuel ending
        shared->player.items[T_FUEL].power = 0;
        break;
      case 'F': // max power in active
        shared->player.active().power = 1600;
        break;
     #endif
    }
    if (dx != 0 || dy != 0)
      fuelRate += manuverPenalty;
  }
  else
  {
    switch (opening)
    {
      case 9:
      case 8:
      case 7:
      case 6:
      case 5:
        dx = shared->zone.scrollx * -1;
        dy = shared->zone.scrolly * -1;
        break;
      case 4:
      case 3:
      case 2:
      case 1:
        if (shared->zone.special != 1)
        {
          dx = shared->zone.scrollx;
          dy = shared->zone.scrolly;
        }
        break;
    }
    opening--;
  }

  //smoke clouds before explosion
  if (shared->player.items[T_HITS].power < 0)
  {
    shared->player.items[T_HITS].power--;
    dx = sign((13 + shared->zone.scrollx * -3) - x);
    dy = sign((13 + shared->zone.scrolly * -3) - y);

    if (shared->player.items[T_HITS].power < -20)
    {
      if (shared->zone.scrolly == 0)
        dy = random(3)-1;
      else
        dx = random(3)-1;
    }

    if ((x == shared->zone.maxleft && dx == -1)
     || (x == shared->zone.maxright && dx == 1))
      dx = 0;
    if ((y == shared->zone.maxtop && dy == -1)
     || (y == shared->zone.maxbottom && dy == 1))
      dy = 0;

    Sprite *cloud = sm->findFreeSprite();
    if (cloud)
    {
      sm->wall(cloud, shared->zone.scrollx, shared->zone.scrolly,
               LIGHTGRAY, random(3)+176);
      cloud->x = shipFire->x;
      cloud->y = shipFire->y;
    }

    if (shared->player.items[T_HITS].power % 2 == 0)
      speaker.play(70);
    if (shared->player.items[T_HITS].power <= -40)
      subgamestat = 1;

    shared->zone.prob[ST_EXPLODE] += 5;

  } // end smoke clouds
}

void FlightModel::doBoss(void)
{
  if (shared->zone.special == 3)
  {
    sm->bossSet(distance);
    sm->catchPower();
  }
  else 
    distance--;

  if (distance <= 0 && shared->zone.special != SL_BOSS 
   && shared->player.items[T_HITS].power >= 0 && subgamestat == 0)
    subgamestat = 2;
  else if (distance <= 0 && shared->zone.special == SL_BOSS)
  {
    shared->zone.special = SL_POSTBOSS;
    distance = 40;

    sm->killBoss();

    for (int a = 0;a < NUMSPRITETYPE;a++)
    {
      shared->zone.prob[a] = 0;
    }

    int bob = shared->zone.bcolor;
    shared->zone.bcolor = LIGHTGRAY;
    blankBackground();

    speaker.play(100);
    delay(200);
    speaker.play(75);
    delay(200);
    speaker.play(50);
    delay(200);
    speaker.stop();

    shared->zone.bcolor = bob;
    blankBackground();
    
  }

  // post-boss
  if (shared->zone.special == SL_POSTBOSS)
    sm->home(ST_POWER, x, y);
}

void FlightModel::draw(void)
{
  gotoxy(32, 25);
  cset(WHITE);
  cprintf("%d  ", distance);

  if (shared->player.active().iAm == T_LASER
   && (shared->player.active().useable & (CU_OWNED | CU_FLIGHT))
   && shared->player.active().power%50 == 49)
    shipNose->bcolor = BLUE;
  else
    shipNose->bcolor = -1;

  sm->draw();
  sm->redraw(shipNose);
  sm->redraw(shipBody);
  sm->redraw(shipFire);

  shared->player.updateItems(CU_FLIGHT);

  shared->player.expBar();

  x += dx;
  y += dy;

}
  
void FlightModel::useItem(ItemType *i)
{
  if (i == NULL)
    i = &(shared->player.active());

  if ((i->power < i->pUse && i->iAm != T_RANDOM) 
   || !(i->useable & (CU_FLIGHT | CU_OWNED)) )
  {
    speaker.play(50);
    return;
  }

  Sprite *s;

  i->genericUse(shared->zone);

  switch (i->iAm)
  {
    case T_GAINITEM:
      shared->player.gainItem();
      i->usePower();
      break;
    case T_PHASE:
    case T_SHIELD:
      itemOnOff(i);
      invinc = 2;
      break;
    case T_LASER:
      if (i->power%50 == 49)
      {
        speaker.play(100);
        s = sm->findFreeSprite();
        if (s)
          sm->laser(s, x, y);
        i->usePower();
      }
      else
      {
        speaker.play(50);
        i->power -= i->power%50;
      }
      break;
    case T_SUPERFR:
      speaker.play(500);
      sm->reflectAll(i);
      i->soil(ALOT);
      break;
    case T_WARP:
      speaker.play(500);
      if (i->on)
        ;//i->on = 0;
      else
        i->on = 1;
      itemOnOff(i);
      break;
    case T_SEED:
      shared->zone.prob[ST_POWER]++;
      sm->spreadEffect(x, y, ST_POWER);
      i->usePower();
      break;
    case T_BOMBARD:
      shared->zone.prob[ST_METEOR] *= 0.8;
      shared->zone.prob[ST_METEOR] += shared->zone.prob[ST_COMET] * 0.2;
      shared->zone.prob[ST_COMET] *= 0.8;
      shared->zone.prob[ST_WALL] *= 0.8;
      shared->zone.prob[ST_MINE] *= 0.8;
      i->usePower();
      sm->spreadEffect(x, y, ST_BOMB);
      break;
    case T_RANDOM:
      speaker.play(300);
      shared->player.experience = 0;
      shared->player.expBar(ALOT);
      shared->player.clearMessage(1);
      break;

  }
}

void FlightModel::randomEffect(int what, int subWhat)
{
  int r;

  switch(what)
  {
    case T_PHASE: // invincible
      shipNose->reColor(YELLOW);
      shipBody->reColor(YELLOW);
      invincTime = 40;
      shared->player.invincBar(ALOT, invincTime);
      shared->player.setMessage("NahNanaBooBoo", 10);
      break;
    case T_HITS:
      shared->player.items[what].power += shared->player.items[what].pUse;
      shared->player.items[what].soil(ALOT);
      shared->player.setMessage("I Needed That", 10);
      break;
    case T_LASER: // gain power
      shared->player.items[subWhat].take();
      shared->player.items[subWhat].soil(ALOT);
      break;
    case T_FUEL:
      shared->player.items[what].take();
      shared->player.items[what].soil(ALOT);
      break;
    case T_SHIELD: // force wall
      speaker.play(1000);
      int a;
      Sprite* s;
      switch(shared->zone.scrollx+(shared->zone.scrolly*2))
      {
        case 1: a = shared->zone.maxleft+5; break;
        case -1: a = shared->zone.maxright-5; break;
        case 2: a = shared->zone.maxtop+5; break;
        case -2: a = shared->zone.maxbottom-5; break;
      }
      if (shared->zone.scrolly == 0)
      {
        for (int b = shared->zone.maxtop;b <= shared->zone.maxbottom;b++)
        {
          s = sm->findFreeSprite();
          if (s)
            sm->fwall(s, a, b);
        }
      }
      else
      {
        for (int b = shared->zone.maxleft;b <= shared->zone.maxright;b++)
        {
          s = sm->findFreeSprite();
          if (s)
            sm->fwall(s, b, a);
        }
      }
      shared->player.setMessage("Stay Away!   ", 10);
      break;
    case T_RANDOM: // extra spins
      shared->player.plus(2);
      shared->player.select().soil(ALOT);
      shared->player.items[what].soil(ALOT);
      break;
    case T_WARP:
      if (distance > 30)
        warpOn(distance - 30);
      else
        warpOn(1);
      shared->player.setMessage("Outta Here!  ", 50);
      break;
    case T_SEED: // asteroids to spins
      speaker.play(500);
      sm->powerAll(); 
      shared->player.setMessage("Oh Yeah.     ", 10);
      break;
    case T_SUPERFR:
      sm->reflectAll(NULL);
      shared->player.setMessage("Take That!   ", 10);
      break;
    default:
    case T_SLOWDOWN:
      shared->player.items[what].power
        += shared->player.items[what].pUse;
      useItem(&(shared->player.items[what]));
      shared->player.setMessage("Much Better  ", 10);
      break;
    case T_GAINITEM:
      shared->player.gainItem();
      shared->player.setMessage("What's This? ", 10);
      break;
    case T_PING:
      sm->pingEffect();
      shared->player.setMessage("I See You   ", 10);
      break;
	
  }
}

void FlightModel::itemOnOff(ItemType *i, int calls)
{
  i->genericOnOff();

  if (i->on)
  {
    switch(i->iAm)
    {
      case T_PHASE:
        shipNose->reColor(YELLOW);
        shipBody->reColor(YELLOW);
        break;
      case T_SHIELD:
        shipNose->flashTo(BLUE);
        shipBody->flashTo(BLUE);
        if (shared->player.items[T_PHASE].on && calls < 2)
          itemOnOff(&(shared->player.items[T_PHASE]), 2);
        break;
      default: break;
    }
  }
  else
  {
    switch(i->iAm)
    {
      case T_PHASE:
        shipNose->reColor(LIGHTGRAY);
        shipBody->reColor(LIGHTGRAY);
        if (shared->player.items[T_SHIELD].on && calls < 2)
          itemOnOff(&(shared->player.items[T_SHIELD]), 2);
        if (invincTime > 0)
        {
          shipNose->reColor(YELLOW);
          shipBody->reColor(YELLOW);
        }
        break;
      case T_SHIELD:
        shipNose->flashTo(LIGHTGRAY);
        shipBody->flashTo(LIGHTGRAY);
        if (calls < 2)
          itemOnOff(&(shared->player.items[T_PHASE]), 2);
        break;
      default: break;
    }
  }
}

void FlightModel::blankBackground(void)
{
  cset(RED, shared->zone.bcolor);
  for (int bx = shared->zone.maxleft;bx <= shared->zone.maxright;bx++)
  {
    for (int by = shared->zone.maxtop;by <= shared->zone.maxbottom;by++)
    {
      gotoxy(bx, by);
      putch(' ');
    }
  }
}

void FlightModel::warpOn(int wt)
{
  if (warpTime < 0)
  {
    warpTime++; // gets it to zero so below statement works OK
    warpSound = 200;
    wsInc = 1;
    invincTime = 10;
    shared->player.invincBar(ALOT, invincTime);
    shipNose->reColor(YELLOW);
    shipBody->reColor(YELLOW);
    speed = 100;
    oldbc = shared->zone.bcolor;
    shared->zone.bcolor = LIGHTGRAY;
    blankBackground();
  }

  warpTime += wt;
}
