//the ASTER@IDS game and related functions

#include "gamefunc.h"
#include "escfp23x.h"
#include "mapfunc.h"
#include "amap.h"
#include "asprite.h" // sprite constants

enum lines {L_TB = 179, L_LR = 196, L_LT = 217, L_LB = 191,
  L_RT = 192, L_RB = 218};

enum relative {R_UP = 0, R_RIGHT = 1, R_DOWN = 2, R_LEFT = 3};

enum screen_type {SC_SCANNER = 1, SC_RADAR = 0,
                  SC_TOSCANNER = 3, SC_TORADAR = 2};

const char whichLine[4][4] = {
//          to
// from     R_UP   R_RIGHT  R_DOWN  R_LEFT
/*R_UP   */{L_TB,  L_RB,    0,      L_LB  },
/*R_RIGHT*/{L_LT,  L_LR,    L_LB,   0     },
/*R_DOWN */{0,     L_RT,    L_TB,   L_LT  },
/*R_LEFT */{L_RT,  0,       L_RB,   L_LR  } };

const int X = 0, Y = 1;
const int dirDelta[4][2] = {
/*R_UP   */{0, -1},
/*R_RIGHT*/{1, 0},
/*R_DOWN */{0, 1},
/*R_LEFT */{-1, 0} };

int canEnter[NUMZONES][4];
int canExit[NUMZONES][4];

void MapModel::initialize(void)
{
  shared = NULL;
  x = 0;
  y = 5;
  relnew = relcur = R_RIGHT;

  for (int a = 0;a < NUMZONES;a++)
  {
    for (int b = 0;b < 4;b++)
    {
      canEnter[a][b] = 1;
      canExit[a][b] = 1;
    }
  }

  for (int a = 0;a < 4;a++)
    canEnter[Z_SOLID][a] = 0;

  canEnter[Z_TOP][R_UP] = 0;
  canEnter[Z_LEFT][R_LEFT] = 0;
  canEnter[Z_RIGHT][R_RIGHT] = 0;
  canEnter[Z_BOTTOM][R_DOWN] = 0;

  // if you somehow get stuck in a solid square,
  //  it won't stop you from leaving

  canExit[Z_TOP][R_DOWN] = 0;
  canExit[Z_LEFT][R_RIGHT] = 0;
  canExit[Z_RIGHT][R_LEFT] = 0;
  canExit[Z_BOTTOM][R_UP] = 0;
}

// zone selection function
int MapModel::start(Shared *s)
{
  shared = s;

  subgamestat = 0;

  while (getkey())
    ;

  cset(BLUE);
  drawframe();

  cheat = 0;

  switchToRadar();

  shared->player.updateInfo(ALOT);

  shared->zone.powerMod = 0;
  shared->zone.meteorMod = 0;

  frameControl.set(36);
  frameControl.sync();

  shared->player.active().soil(ALOT);
  shared->player.select().soil(ALOT);
  shared->player.updateInfo(ALOT);

  while(subgamestat == 0)
    run();

  return subgamestat;
}

void MapModel::run(void)
{
  float range;

  if (shared->player.items[T_RADAR].pUse == 0)
  {
    cputs("blah");
    getch();
  }
  range = (float)shared->player.items[T_RADAR].power
        / (float)shared->player.items[T_RADAR].pUse;
  range *= range;

  nearReveal(x, y, range);

  control();

  update();

  speaker.stop();
}

void MapModel::control(void)
{
 int key = getkey();

 genericControl(key);

 if (screen == SC_SCANNER)
   scannerControl(key);
 else
   radarControl(key);
}  

void MapModel::update(void)
{
  switch(screen)
  {
    case SC_SCANNER:
      updateScanner();
      break;
    case SC_RADAR:
      updateRadar();
      break;
    case SC_TOSCANNER:
      switchToScanner();
      break;
    case SC_TORADAR:
      switchToRadar();
      break;
  }
  shared->player.updateItems(CU_MAP);
}

void MapModel::genericControl(int key)
{
  int t;

  switch (key)
  {
    case ESCAPE:
      pause(subgamestat);
      shared->player.updateInfo(ALOT);
      break;
    case 'b':
    case 'B':
      speaker.playFor(700, 50);
      if (shared->player.playMode == PM_NEW)
        shared->player.IBSelect(&(shared->player.itemB), CU_MAP);
      else
        useItem(&(shared->player.items[T_RANDOM]));
      break;
    case 'v':
    case 'V':
      speaker.playFor(700, 50);
      shared->player.IBSelect(&(shared->player.itemA), CU_MAP);
      break;
    case 'u':
    case 'U':
      speaker.play(500);
      useItem();
      break;
  }
}

void MapModel::scannerControl(int key)
{
  switch (key)
  {
    case UP: if (newy > 0) newy--; break;
    case DOWN: if (newy < 9) newy++; break;
    case LEFT: if (newx > 0) newx--; break;
    case RIGHT: if (newx < 19) newx++; break;
    case ENTER:
      if (zone[newx][newy] != 26 && cheat == 3)
      {
        x = newx;
        y = newy;
      }
      break;
    case SPACE: screen = SC_TORADAR; break;
  }
}

void MapModel::radarControl(int key)
{
  switch (key)
  {
    case UP:
      relnew = R_UP;
      break;
    case DOWN:
      relnew = R_DOWN;
      break;
    case LEFT:
      relnew = R_LEFT;
      break;
    case RIGHT:
      relnew = R_RIGHT;
      break;
    case ENTER:
      advance();
      break;
    case SPACE:
      screen = SC_TOSCANNER;
      break;
   #ifdef CHEAT
    case (int)'C':
      if (cheat == 0)
        cheat = 1;
      break;
    case (int)'G':
      if (cheat == 1)
        cheat = 2;
      break;
   case (int)'D':
      if (cheat == 2)
      {
        speaker.play(1000);
        delay(50);
        speaker.stop();
        cheat = 3;
        reveal();
      }
      break;
   #endif
  }

  // corners are shorter.
  if (relnew == relcur)
    shared->zone.length = 610;
  else
    shared->zone.length = 410;

  decideRadarCursor();
}

void MapModel::updateScanner(void)
{
  // flashing cursor
  gotoxy(newx+4, newy+4);
  cset(random(5)+1);
  switchscanner(zone[newx][newy], pic);
  putch(pic);

  // info
  shared->zone.define(zone[newx][newy]);
  shared->player.pProbSet(zone[newx][newy]);
  cset(DARKGRAY, LIGHTGRAY);

  gotoxy(19, 16);
  showspot(newx, newy);

  int freeSpace = 100;

  gotoxy(7, 17);
  //cprintf("%d ", (int)((float)maxm*10.0/4.0));  //  x / 40 * 100
  cprintf("%d ", shared->zone.prob[ST_METEOR]);
  freeSpace -= shared->zone.prob[ST_METEOR];

  gotoxy(7, 18);
  cprintf("%d ", shared->zone.prob[ST_COMET]);
  freeSpace -= shared->zone.prob[ST_COMET];

  gotoxy(7, 19);
  cprintf("%d ", shared->zone.prob[ST_WARP]);
  freeSpace -= shared->zone.prob[ST_WARP];

  gotoxy(7, 20);
  cprintf("%d ", shared->zone.prob[ST_POWER]);
  freeSpace -= shared->zone.prob[ST_POWER];

  gotoxy(7, 21);
  cprintf("%d ", shared->zone.prob[ST_WALL]);
  freeSpace -= shared->zone.prob[ST_WALL];

  gotoxy(7, 22);
  cprintf("%d ", shared->zone.prob[ST_MINE]);
  freeSpace -= shared->zone.prob[ST_MINE];

  gotoxy(7, 23);
  cprintf("%d ", freeSpace);

  gotoxy(14, 24);
  cprintf("%d  ", shared->zone.speed*10);

  frameControl.force();
  gotoxy(newx+4, newy+4);
  cset(switchscanner(zone[newx][newy], pic));
  putch(pic);
  shared->zone.define(zone[newx][newy]);
  shared->player.pProbSet(zone[newx][newy]);
}

void MapModel::updateRadar(void)
{
  gotoxy(x+4, y+4);
  cset(RED);
  putch(linech);

  gotoxy(newx+4, newy+4);
  if (newx < 20 && newx > -1 && newy < 10 && newy > -1)
  {
    cset(GREEN, LIGHTGRAY);
    putch(switchradar(zone[newx][newy]));
  }
  else if (newx == 20 && newy == 5)
  {
    cset(BLUE, LIGHTGRAY);
    putch(216);
  }

  cset(DARKGRAY, LIGHTGRAY);

  // between 1
  gotoxy(21, 16);
  showspot(x, y);

  // between 2
  gotoxy(6, 17);
  int opposite;
  switch(relcur)
  {
    case R_RIGHT:
      opposite = R_LEFT;
      break;
    case R_LEFT:
      opposite = R_RIGHT;
      break;
    case R_UP:
      opposite = R_DOWN;
      break;
    case R_DOWN:
      opposite = R_UP;
      break;
  }

  showspot(x + dirDelta[opposite][X], y + dirDelta[opposite][Y]);

  // travel to next
  gotoxy(10, 18);
  showspot(newx, newy);

  // through
  gotoxy(21, 18);
  showspot(x, y);

  // looks
  gotoxy(2, 19);
  showspot(newx, newy);

  gotoxy(11, 19);
  if (newx == 20 && newy == 5)
                      cputs("like an exit. ");
  else
  switch(zone[newx][newy])
  {
    case Z_EMPTY:     cputs("totally empty."); break;
    case Z_SLOWROCK:
    case Z_SLOWCOMET:
    case Z_SLOWWARP:
    case Z_SLOWWALL:
    case Z_SLOWMINE:
    case Z_SLOWHUNK:  cputs("fairly clear. "); break;
    case Z_MEDROCK:
    case Z_MEDCOMET:
    case Z_MEDWARP:
    case Z_MEDWALL:
    case Z_MEDMINE:
    case Z_MEDHUNK:   cputs("crowded.      "); break;
    case Z_FASTROCK:
    case Z_FASTCOMET:
    case Z_FASTWARP:
    case Z_FASTWALL:
    case Z_FASTMINE:
    case Z_TOP:
    case Z_RIGHT:
    case Z_LEFT:
    case Z_BOTTOM:
    case Z_FASTHUNK:
                      cputs("very tight.   "); break;
    case Z_DISTROCK:
    case Z_DISTWARP:
    case Z_DISTCOMET: cputs("electrostatic."); break;
    case Z_SOLID:     cputs("solid.        "); break;
    case Z_SECRET:    cputs("different.    "); break;
    case Z_BOSS:      cputs("very strange. "); break;
    default:          cputs("unexplored.   "); break;
  }

  gotoxy(2, 20);
  if (shared->player.items[T_FUEL].power > 0)
  {
    cprintf("I have %d units of   ", shared->player.items[T_FUEL].power);
    gotoxy(2, 21);
    cputs("fuel to get past here. ");
  }
  else
  {
    cputs("My only hope is to     ");
    gotoxy(2, 21);
    cputs("reach traveled space.  ");
  }

  gotoxy(2, 23);
  if (shared->player.items[T_HITS].pUse == 0)
  {
    cputs("blah 2");
    getch();
  }
  switch(shared->player.items[T_HITS].power
         / shared->player.items[T_HITS].pUse)
  {
    default: cputs("neigh impenetrable.    "); break;
    case 7:
    case 6:
    case 5:  cputs("armored like a rock.   "); break;
    case 4:
    case 3:  cputs("in excellent condition."); break;
    case 2:  cputs("holding up well.       "); break;
    case 1:  cputs("begining to worry me.  "); break;
    case 0:  cputs("badly damaged.         "); break;
  }

  gotoxy(2, 24);
  if (zone[x][y] == Z_SLOWWALL)
    cputs("This is an alien base.");
  else if (zone[x][y] == Z_BOSS)
    cputs("My path is blocked.   ");
  else if (x == 19 && y == 5)
    cputs("I am almost home.     ");
  else if (shared->player.items[T_HITS].power == 0)
    cputs("I fear for my life.    ");

  frameControl.force();
  //erase
  gotoxy(x+4, y+4);
  cset(GREEN);
  putch(switchradar(zone[x][y]));

  gotoxy(newx+4, newy+4);
  if (newx < 20 && newx > -1 && newy < 10 && newy > -1)
    putch(switchradar(zone[newx][newy]));
  else if (newx == 20 && newy == 5)
  {
    cset(BLUE);
    putch(216);
  }
  frameControl.force();
}

void MapModel::switchToRadar(void)
{
  cset(GREEN);
  gotoxy(7, 15);
  cputs("SCANNER  ");
  cset(BLACK, GREEN);
  cputs("RADAR");

  cset(GREEN);
  for (int a = 0;a < 20;a++)
  {
    for (int b = 0;b < 10;b++)
    {
      gotoxy(a+4, b+4);
      putch(switchradar(zone[a][b]));
    }
  }

  window(2, 16, 24, 24);
  cset(DARKGRAY, LIGHTGRAY);
  gotoxy(1, 9);
  cputs("I am between zones     "); delay(50);
  cputs("and   .  I will go next"); delay(50);
  cputs("to zone    through   . "); delay(50);
  cputs("   looks               "); delay(50);
  cputs("I have      units of   "); delay(50);
  cputs("fuel to get past here  "); delay(50);
  cputs("The ship\'s hull is     "); delay(50);
  cputs("                       "); delay(50);
  cputs("                      "); delay(50);
  window(1, 1, 40, 25);

  shared->player.invincBar(ALOT);

  screen = SC_RADAR;

  decideRadarCursor();
}

void MapModel::switchToScanner(void)
{
  cset(BLACK, GREEN);
  gotoxy(7, 15);
  cputs("SCANNER");
  cset(GREEN);
  cputs("  RADAR");

  for (int a = 0;a < 20;a++)
  {
    for (int b = 0;b < 10;b++)
    {
      gotoxy(a+4, b+4);
      cset(switchscanner(zone[a][b], pic));
      putch(pic);
    }
  }

  window(2, 16, 24, 24);
  cset(DARKGRAY, LIGHTGRAY);
  gotoxy(1, 8);
  cputs("    Scan of zone       "); delay(50);
  cputs("        %asteroids     "); delay(50);
  cputs("        %comets        "); delay(50);
  cputs("        %warps         "); delay(50);
  cputs("        %space junk    "); delay(50);
  cputs("        %walls         "); delay(50);
  cputs("        %mines         "); delay(50);
  cputs("        %free          "); delay(50);
  cputs("     speed:           "); delay(50);
  window(1, 1, 40, 25);

  shared->player.invincBar(ALOT);

  screen = SC_SCANNER;

  if (newx < 0 || newx > 19)
    newx = 10;
  if (newy < 0 || newy > 9)
    newy = 5;
}

void MapModel::advance(void)
{
  int exiting, entering;
  if (newx == 20 && newy == 5)
    entering = 0;
  else
    entering = zone[newx][newy];
  exiting = zone[x][y];
  if (canExit[exiting][relnew] && canEnter[entering][relnew])
  {
    subgamestat = MRV_FLIGHT;
    shared->zone.scrollx = 0;
    shared->zone.scrolly = 0;
    switch (relnew)
    {
      case R_UP:
        if (exiting != Z_TOP)
          shared->zone.scrolly = 1;
        else if (relcur == R_RIGHT)
          shared->zone.scrollx = -1;
        else 
          shared->zone.scrollx = 1;
        break;
      case R_DOWN:
        if (exiting != Z_BOTTOM)
          shared->zone.scrolly = -1;
        else if (relcur == R_LEFT)
          shared->zone.scrollx = 1;
        else 
          shared->zone.scrollx = -1;
        break;
      case R_LEFT:
        if (exiting != Z_LEFT)
          shared->zone.scrollx = 1;
        else if (relcur == R_UP)
          shared->zone.scrolly = 1;
        else 
          shared->zone.scrolly = -1;
        break;
      case R_RIGHT:
        if (exiting != Z_RIGHT)
          shared->zone.scrollx = -1;
        else if (relcur == R_DOWN)
          shared->zone.scrolly = -1;
        else 
          shared->zone.scrolly = 1;
        break;
    }
    shared->zone.define(exiting);
    shared->player.pProbSet(exiting);

    if (exiting == Z_SLOWWALL)
      shared->player.setMessage("Walls?       ", 20);
    else if (exiting == Z_BOSS)
      shared->player.setMessage("You in my way", 20);

    x = newx;
    y = newy;
    relcur = relnew;

    if (x == 20 && y == 5)
      subgamestat = MRV_FINAL;
  }
}

void MapModel::decideRadarCursor(void)
{
  // reverse if neccisary.
  if (whichLine[relcur][relnew] == 0)
  {
    relcur = relnew;
    x += dirDelta[relcur][X];
    y += dirDelta[relcur][Y];
  }

  // check to see if line would be off the edge
  switch(relcur)
  {
    case R_LEFT:
      if (x < 0)
      {
        relnew = relcur = R_RIGHT;
        x += dirDelta[relcur][X];
        y += dirDelta[relcur][Y];
      }
      break;
    case R_RIGHT:
      if (x >= MAPXSIZE)
      {
        relnew = relcur = R_LEFT;
        x += dirDelta[relcur][X];
        y += dirDelta[relcur][Y];
      }
      break;
    case R_UP:
      if (y < 0)
      {
        relnew = relcur = R_DOWN;
        x += dirDelta[relcur][X];
        y += dirDelta[relcur][Y];
      }
      break;
    case R_DOWN:
      if (y >= MAPYSIZE)
      {
        relnew = relcur = R_UP;
        x += dirDelta[relcur][X];
        y += dirDelta[relcur][Y];
      }
      break;
  }

  // position the next area cursor

  // stage 1: get initial guess
  newx = x + dirDelta[relnew][X];
  newy = y + dirDelta[relnew][Y];

  // allow the victory square
  if (newx == 20 && newy == 5)
  {
    linech = whichLine[relcur][relnew];
    return;
  }

  // otherwise, check against walls; turn 90 deg. if not.
  switch(relnew)
  {
    case R_LEFT:
      if (newx < 0)
        relnew = R_UP;
      break;
    case R_RIGHT:
      if (newx >= MAPXSIZE)
        relnew = R_DOWN;
      break;
    case R_UP:
      if (newy < 0)
        relnew = R_RIGHT;
      break;
    case R_DOWN:
      if (newy >= MAPYSIZE)
        relnew = R_LEFT;
      break;
  }

  // stage two, figure new position
  newx = x + dirDelta[relnew][X];
  newy = y + dirDelta[relnew][Y];
  // and make sure that's not off the edge
  switch(relnew)
  {
    case R_LEFT:
      if (newx < 0)
        relnew = R_RIGHT;
      break;
    case R_RIGHT:
      if (newx >= MAPXSIZE)
        relnew = R_LEFT;
      break;
    case R_UP:
      if (newy < 0)
        relnew = R_DOWN;
      break;
    case R_DOWN:
      if (newy >= MAPYSIZE)
        relnew = R_UP;
      break;
  }
  // stage 3: final position
  newx = x + dirDelta[relnew][X];
  newy = y + dirDelta[relnew][Y];

  // figure out the line pic
  linech = whichLine[relcur][relnew];
}       

void MapModel::hide(void)
{
  for (int my = 0;my < MAPYSIZE;my++)
  {
    for (int mx = 0;mx < MAPXSIZE;mx++)
    {
      if (zone[mx][my] > -1)
        zone[mx][my] -= 1000;
    }
  }
}

void MapModel::reveal(void)
{
  for (int my = 0;my < MAPYSIZE;my++)
  {
    for (int mx = 0;mx < MAPXSIZE;mx++)
    {
      if (zone[mx][my] < 0)
        zone[mx][my] += 1000;
    }
  }
}

void MapModel::nearReveal(int rx, int ry, float range)
{
  float d;

  for (int my = 0;my < MAPYSIZE;my++)
  {
    for (int mx = 0;mx < MAPXSIZE;mx++)
    {
      d = (mx-rx)*(mx-rx) + (my-ry)*(my-ry);
      if ( zone[mx][my] < 0 && d <= range)
      {
        zone[mx][my] += 1000;
        refreshSquare(mx, my);
      }
    }
  }
}

// print the grid location
inline void MapModel::showspot(int sx, int sy)
{
  if (sx < 0 || sx >= MAPXSIZE || sy < 0 || sy >= MAPYSIZE)
    cputs("??");
  else
    cprintf("%d%c", sy, 'A' + (char)sx);
}

void MapModel::refreshSquare(int sx, int sy)
{
  int pic;
  switch(screen)
  {
    case SC_RADAR:
      cset(GREEN);
      gotoxy(sx+4, sy+4);
      putch(switchradar(zone[sx][sy]));
      break;
    case SC_SCANNER:
      gotoxy(sx+4, sy+4);
      cset(switchscanner(zone[sx][sy], pic));
      putch(pic);
  }
}

void MapModel::useItem(ItemType *i)
{
  if (i == NULL)
    i = &(shared->player.active());
  int rx, ry, a;
  float range;

  if ((i->power < i->pUse && i->iAm != T_RANDOM) 
   || !(i->useable & (CU_MAP | CU_OWNED)) )
  {
    speaker.play(50);
    return;
  }

  i->genericUse(shared->zone);

  switch (i->iAm)
  {
    case T_GAINITEM:
      shared->player.gainItem();
      i->usePower();
      break;
    case T_WARP:
      if (screen == SC_RADAR && i->power >= shared->zone.length * i->pUse)
      {
        i->power -= i->pUse * shared->zone.length;
        advance();
        subgamestat = MRV_SKIP; // skip zone
        i->soil(ALOT);
      }
      else
        speaker.play(50);
      break;
    case T_RANDSQ:
      for (a = 0;a < 1000;a++)
      {
        rx = random(MAPXSIZE);
        ry = random(MAPYSIZE);
        if (zone[rx][ry] < 0)
        {
          zone[rx][ry] += 1000;
          i->usePower();
          refreshSquare(rx, ry);
          break;
        }
      }
      break;
    case T_CHOOSESQ:
      if (zone[newx][newy] < 0)
      {
        zone[newx][newy] += 1000;
        i->usePower();
        refreshSquare(newx, newy);
      }
      break;
    case T_PING:
      i->usePower();
      if (shared->player.items[T_RADAR].pUse == 0)
      {
        cputs("blah 3");
        getch();
      }
      range = (float)shared->player.items[T_RADAR].power
            / (float)shared->player.items[T_RADAR].pUse;
      range += 2.0;
      range *= range;
      nearReveal(x, y, range);
      break;
    case T_BOMBARD:
      reduceSquare(newx, newy);
      i->usePower();
      break;
    case T_SEED:
      shared->zone.powerMod++;
      i->usePower();
      break;
  }

}

void MapModel::reduceSquare(int wx, int wy)
{
  int wasHidden = 0;

  if (zone[wx][wy] < 0)
  {
    wasHidden = 1;
    zone[wx][wy] == 1000;
  }

  switch(zone[wx][wy])
  {
    case Z_EMPTY:
    case Z_SLOWROCK:
      zone[wx][wy] = Z_EMPTY;
      break;
    case Z_MEDROCK:
    case Z_SLOWCOMET:
    case Z_SLOWWARP:
    case Z_SLOWWALL:
    case Z_SLOWMINE:
      zone[wx][wy] = Z_SLOWROCK;
      break;
    case Z_FASTROCK:
      zone[wx][wy] = Z_MEDROCK;
      break;
    case Z_MEDCOMET:
      zone[wx][wy] = Z_SLOWCOMET;
      break;
    case Z_FASTCOMET:
      zone[wx][wy] = Z_MEDCOMET;
      break;
    case Z_MEDWARP:
      zone[wx][wy] = Z_SLOWWARP;
      break;
    case Z_FASTWARP:
      zone[wx][wy] = Z_MEDWARP;
      break;
    case Z_MEDWALL:
      zone[wx][wy] = Z_SLOWWALL;
      break;
    case Z_FASTWALL:
      zone[wx][wy] = Z_MEDWALL;
      break;
    case Z_MEDMINE:
      zone[wx][wy] = Z_SLOWMINE;
      break;
    case Z_FASTMINE:
      zone[wx][wy] = Z_MEDMINE;
      break;
    case Z_DISTROCK:
    case Z_SLOWHUNK:
      zone[wx][wy] = Z_FASTROCK;
      break;
    case Z_DISTWARP:
      zone[wx][wy] = Z_FASTWARP;
      break;
    case Z_DISTCOMET:
      zone[wx][wy] = Z_FASTCOMET;
      break;
    case Z_SOLID:
    case Z_TOP:
    case Z_RIGHT:
    case Z_LEFT:
    case Z_BOTTOM:
      zone[wx][wy] = Z_FASTHUNK;
      break;
    case Z_MEDHUNK:
      zone[wx][wy] = Z_SLOWHUNK;
      break;
    case Z_FASTHUNK:
      zone[wx][wy] = Z_MEDHUNK;
      break;
    case Z_SECRET:
      zone[wx][wy] = Z_BOSS;
      break;
    default:
      break;
  }

  if (wasHidden)
    zone[wx][wy] -= 1000;
}
