#include <stdio.h>
#include "gamefunc.h"
#include "asprite.h"
#include "const.h" // ASCII pic constants

void Sprite::initialize(void)
{
  id = ST_SPECIAL;
  active = ACT_DEAD;
  react = RA_NONE;
  priority = DP_NORMAL;
  x = 1;
  y = 1;
  dx = 0;
  dy = 0;
  for (int a = 0;a < 0;a++)
  {
    pic[a] = 'a' + a;
    scolor[a] = a;
  }
  bcolor = -1;

  slave = NULL;
}

// bounce an object off
void Sprite::bounceme(int bdx, int bdy)
{
  int a;
  if (bdx == 0 && bdy == 0)
  {
    dx *= -1;
    dy *= -1;
    if (dx == 0)
      dx = random(2) ? 1 : -1;
    if (dy == 0)
      dy = random(2) ? 1 : -1;
    return;
  }

  dx = bdx;
  dy = bdy;
  if (bdx != 0 && bdy != 0)
  {
    dx *= -1;
    dy *= -1;
    a = random(3);
    if (a == 0)
      dx = 0;
    else if (a == 1)
      dy = 0;
  }
  else if (bdx != 0 && bdy == 0)
  {
    dx *= -1;
    dy = random(3)-1;
  }
  else if (bdy != 0 && bdx == 0)
  {
    dy *= -1;
    dx = random(3)-1;
  }
  else
  {
    dx = random(2) ? 1 : -1;
    dy = random(2) ? 1 : -1;
  }
}

void Sprite::bossSet(int bp, int bfc, int bbc)
{
    for (int a = 0;a < 4;a++)
    {
      pic[a] = bp;
      scolor[a] = bfc;
    }
    bcolor = bbc;
}

SpriteManager::SpriteManager(int n)
  : numSprites(n), frame(0), wallRate(1),
    rebuilds(0), found(0), notfound(0), powers(0), instants(0), bossHit(10)
{
  sprite = new Sprite[numSprites];

  if (sprite == NULL)
  {
    cputs ("unable to allocate sprites\r\n");
    if (getch() == 'q')
      exit(1);
    numSprites = 0;
  }

  safe.initialize(23, 23, 2, 2);
}

//common assignments
void SpriteManager::aveassign(Sprite *b, int bdx, int bdy)
{
  switch (bdx + (bdy*2))
  {
    case 1:
      b->x = shared->zone.maxleft;
      b->y = random(shared->zone.maxbottom-shared->zone.maxtop+1)+shared->zone.maxtop;
      break;
    case -1:
      b->x = shared->zone.maxright;
      b->y = random(shared->zone.maxbottom-shared->zone.maxtop+1)+shared->zone.maxtop;
      break;
    case 2:
      b->x = random(shared->zone.maxright-shared->zone.maxleft+1)+shared->zone.maxleft;
      b->y = shared->zone.maxtop;
      break;
    case -2:
      b->x = random(shared->zone.maxright-shared->zone.maxleft+1)+shared->zone.maxleft;
      b->y = shared->zone.maxbottom;
      break;
  }
  b->active = ACT_ALIVE;
  b->dx = bdx;
  b->dy = bdy;
}

//makes 3 objects the ship
void SpriteManager::ship3(Sprite *s1, Sprite *s2, Sprite *s3, int ax, int ay)
{
  s1->id = ST_SHIPNOSE;
  s1->active = ACT_ALIVE;
  s1->x = ax;
  s1->y = ay;
  s1->react = RA_NONE;
  s1->priority = DP_IMPORTANT;

  s2->id = ST_SHIPBODY;
  s2->active = ACT_ALIVE;
  s2->x = ax + shared->zone.scrollx;
  s2->y = ay + shared->zone.scrolly;
  s2->react = RA_NONE;
  s2->priority = DP_IMPORTANT;

  s3->id = ST_SHIPFIRE;
  s3->active = ACT_ALIVE;
  s3->x = ax + shared->zone.scrollx*2;
  s3->y = ay + shared->zone.scrolly*2;
  s3->react = RA_NONE;
  s3->priority = DP_IMPORTANT;

  for (int a = 0;a < 4;a++)
  {
    s1->scolor[a] = LIGHTGRAY;
    s2->scolor[a] = LIGHTGRAY;
    s2->pic[a] = ASC_SOLID;

    switch (shared->zone.scrollx + (shared->zone.scrolly*2))
    {
      case -1: s1->pic[a] = 16; break;
      case 1: s1->pic[a] = 17; break;
      case -2: s1->pic[a] = 31; break;
      case 2: s1->pic[a] = 30; break;
    }
  }

  switch (shared->zone.scrollx +(shared->zone.scrolly*2))
  {
    case 1: s3->pic[0] = 221; s3->scolor[0] = RED;
      s3->pic[1] = 16; s3->scolor[1] = YELLOW;
      s3->pic[2] = 196; s3->scolor[2] = WHITE;
      s3->pic[3] = 16; s3->scolor[3] = YELLOW; break;
    case -1: s3->pic[0] = 222; s3->scolor[0] = RED;
       s3->pic[1] = 17; s3->scolor[1] = YELLOW;
       s3->pic[2] = 196; s3->scolor[2] = WHITE;
       s3->pic[3] = 17; s3->scolor[3] = YELLOW; break;
    case 2: s3->pic[0] = 223; s3->scolor[0] = RED;
      s3->pic[1] = 31; s3->scolor[1] = YELLOW;
      s3->pic[2] = 179; s3->scolor[2] = WHITE;
      s3->pic[3] = 31; s3->scolor[3] = YELLOW; break;
    case -2: s3->pic[0] = 220; s3->scolor[0] = RED;
       s3->pic[1] = 30; s3->scolor[1] = YELLOW;
       s3->pic[2] = 179; s3->scolor[2] = WHITE;
       s3->pic[3] = 30; s3->scolor[3] = YELLOW; break;
  }
}

//makes an object a power-up
void SpriteManager::powrup(Sprite *s, int adx, int ady,
                              int all, int special, int force)
{
  if (adx != 0 && ady != 0)
  {
    if (random(2) == 0) adx = 0;
    else ady = 0;
  }

  if (adx == 0 && ady == 0)
  {
    adx = shared->zone.scrollx;
    ady = shared->zone.scrolly;
  }


  if (all)
    aveassign(s, adx, ady);

  int c;

  if (force)
    s->id = force;
  else if (random(100) < shared->zone.instantProb && special == 0)
    s->id = ST_INSTANT;
  else
    s->id = ST_POWER;

  if (special == 0)
  {
    int r = random(4);
    if (s->id == ST_POWER)
    {
      s->pic[r] = ASC_SW_SE;
      s->pic[(r+1)%4] = (int)'/';
      s->pic[(r+2)%4] = ASC_SN_SS;
      s->pic[(r+3)%4] = (int)'\\';
    }
    else
    {
      s->pic[r] = ASC_TRIANGLE_N;
      s->pic[(r+1)%4] = ASC_TRIANGLE_W;
      s->pic[(r+2)%4] = ASC_TRIANGLE_S;
      s->pic[(r+3)%4] = ASC_TRIANGLE_E;
    }

    for (int a = 0;a < 4;a++)
    {
      c = random(15)+1;
      if (c == shared->zone.bcolor)
        c = 0;
      s->scolor[a] = c;
    }
  }
  else
  {
    for (int a = 0;a < 4;a++)
    {
      s->pic[a] = shared->player.items[special].letter;
      s->scolor[a] = shared->player.items[special].color;
    }
  }

  s->react = RA_POWER + special;

  // don't want things you move twoards hiding things you move away from
  s->priority = DP_POWER;
}

//makes an object an asteroid
void SpriteManager::rock(Sprite *s, int adx, int ady)
{
  int c = random(15)+1;
  if (c == shared->zone.bcolor)
    c = 0;

  if (adx != 0 && ady != 0)
  {
    if (random(2) == 0) adx = 0;
    else ady = 0;
  }

  if (adx == 0 && ady == 0)
  {
    adx = shared->zone.scrollx;
    ady = shared->zone.scrolly;
  }

  aveassign(s, adx, ady);

  for(int a = 0;a < 4;a++)
  {
    s->pic[a] = (int)'@';
    s->scolor[a] = c;
  }

  s->react = RA_BOUNCEHIT;

  s->id = ST_METEOR;
}

//makes 4 objects a comet
void SpriteManager::comet4(Sprite *h, Sprite *t1, Sprite *t2, Sprite *t3,
                              int adx, int ady)
{
  int a;

  if (adx != 0 && ady != 0)
  {
    if (random(2) == 0) adx = 0;
    else ady = 0;
  }

  if (adx == 0 && ady == 0)
  {
    adx = shared->zone.scrollx;
    ady = shared->zone.scrolly;
  }

  aveassign(h, adx, ady);

  for (a = 0;a < 4;a++)
    h->scolor[a] = WHITE;

  h->react = RA_HARDHIT;
  *t1 = *h;
  *t2 = *h;
  *t3 = *h;
  t1->active = ACT_SLAVE;
  t2->active = ACT_SLAVE;
  t3->active = ACT_SLAVE;

  h->id = ST_COMET;
  t1->id = ST_CTAIL1;
  t2->id = ST_CTAIL2;
  t3->id = ST_CTAIL3;

  h->slave = t1;
  t1->slave = t2;
  t2->slave = t3;

  for (a = 0;a < 4;a++)
  {
    h->pic[a] = (int)'@';
    t1->pic[a] = ASC_PATTERNHEAVY;
    t2->pic[a] = ASC_PATTERNMEDIUM;
    t3->pic[a] = ASC_PATTERNLIGHT;
  }
  int r = random(4);
  t1->pic[r] = ASC_SOLID;
  t1->pic[(r+2)%4] = ASC_PATTERNMEDIUM;
  t2->pic[r] = ASC_PATTERNHEAVY;
  t2->pic[(r+2)%4] = ASC_PATTERNLIGHT;
  t3->pic[r] = ASC_PATTERNMEDIUM;
  t3->pic[(r+2)%4] = ' ';
}

//makes an object a star
void SpriteManager::star(Sprite *s)
{
  s->id = ST_STAR;

  s->x = random((shared->zone.maxright-shared->zone.maxleft)-1)
          +shared->zone.maxleft+1;
  s->y = random((shared->zone.maxbottom-shared->zone.maxtop)-1)
          +shared->zone.maxtop+1;
  s->dx = 0;
  s->dy = 0;
  s->active = ACT_ALIVE;

  for(int a = 0;a < 4;a++)
  {
    s->pic[a] = 249 + random(2);
    s->scolor[a] = WHITE;
  }
  s->react = RA_NONE;
  s->priority = DP_BACKGROUND;
}

//makes an object a wall
void SpriteManager::wall(Sprite *s, int adx, int ady, int wcolor, int wpic)
{
  aveassign(s, adx, ady);

  s->id = ST_WALL;

  s->react = RA_HARDHIT;

  for (int a = 0;a < 4;a++)
  {
    s->pic[a] = wpic;
    s->scolor[a] = wcolor;
  }
}

//makes an object a laser
void SpriteManager::laser(Sprite *s, int ax, int ay)
{
  int spic;
  s->dx = shared->zone.scrollx * -1;
  s->dy = shared->zone.scrolly * -1;
  if (shared->zone.scrollx == 0)
  {
    spic = 179;
    s->y = ay;
    s->x = ax;
  }
  else
  {
    spic = 196;
    s->x = ax;
    s->y = ay;
  }
  int r = random(2);
  for (int a = 0;a < 4;a++)
  {
    s->pic[a] = spic;
    if ((r+a+1) % 2 == 0)
      s->scolor[a] = WHITE;
    else
      s->scolor[a] = BLUE;
  }
  s->react = RA_FORCER;
  s->id = ST_LASER;
  s->active = ACT_ALIVE;
}

//makes an object a force wall
void SpriteManager::fwall(Sprite *s, int ax, int ay)
{
  int spic;

  if (shared->zone.scrolly == 0)
    spic = 179;
  else
    spic = 196;

  s->x = ax;
  s->y = ay;
  s->dx = 0;
  s->dy = 0;

  for (int a = 0;a < 4;a++)
  {
    s->pic[a] = spic;
    if ((a+1) % 2 == 0)
      s->scolor[a] = WHITE;
    else
      s->scolor[a] = BLUE;
  }
  s->react = RA_FORCER;
  s->id = ST_LASER;
  s->active = ACT_ALIVE;
}

//makes an object a light
void SpriteManager::light(Sprite *s, int adx, int ady, int lcolor)
{
  aveassign(s, adx, ady);
  s->id = ST_LIGHT;
  s->react = RA_NONE;
  s->priority = DP_BACKGROUND;
  for (int a = 0;a < 4;a++)
    s->pic[a] = 254;

  int r = random(2);

  s->scolor[0] = lcolor + r*8;
  s->scolor[1] = lcolor + r*8;
  s->scolor[2] = lcolor + 8 - r*8;
  s->scolor[3] = lcolor + 8 - r*8;
}

//makes an object a warp
void SpriteManager::warp(Sprite *s, int ax, int ay, int adx, int ady)
{
  int c = random(15)+1;
  if (c == shared->zone.bcolor)
    c = 0;

  if (adx != 0 && ady != 0)
  {
    if (random(2) == 0)
      adx = 0;
    else
      ady = 0;
  }
  if (adx == 0 && ady == 0)
  {
    adx = shared->zone.scrollx;
    ady = shared->zone.scrolly;
  }
  aveassign(s, adx, ady);
  if (ady == 0)
    s->y = ay;
  else
    s->x = ax;
  s->react = RA_WARP;
  int r = random(4);
  s->pic[r] = 'O'; //21;
  s->pic[(r+1)%4] = 'o';//128;
  s->pic[(r+2)%4] = ASC_POINTLARGE;//21;
  s->pic[(r+3)%4] = ASC_CIRCLESMALL;//235;
  for (int a = 0;a < 4;a++)
    s->scolor[a] = c;//random(15)+1;

  s->id = ST_WARP;
  // warps can't be reflected, don't want something hiding them
  s->priority = DP_IMPORTANT;
}

//makes an object a stealth mine
void SpriteManager::stealth(Sprite *s, int adx, int ady)
{
  aveassign(s, adx, ady);

  s->id = ST_MINE;

  for (int a = 0;a < 4;a++)
  {
    s->pic[a] = 4;
    s->scolor[a] = shared->zone.bcolor;
  }
  s->scolor[random(4)] = 15;
  s->react = RA_BOUNCEHIT;
}

//makes an object a boss laser
void SpriteManager::bosslaser(Sprite *s1, Sprite *s2, Sprite *s3)
{
  aveassign(s1, shared->zone.scrollx, shared->zone.scrolly);
  s1->id = ST_BOSSLASER;
  s1->react = RA_BOUNCEHIT;
  for (int a = 0;a < 4;a++)
    s1->pic[a] = 196;
  int r = random(4);
  s1->scolor[r] = RED;
  s1->scolor[(r+1)%4] = WHITE;
  s1->scolor[(r+2)%4] = RED;
  s1->scolor[(r+3)%4] = WHITE;
  s1->x += s1->dx * 2;
  s1->y += s1->dy * 2;

  *s2 = *s1;
  *s3 = *s1;
  s2->active = ACT_SLAVE;
  s3->active = ACT_SLAVE;

  s1->slave = s2;
  s2->slave = s3;
}

void SpriteManager::bossBlock(Sprite *s, int i)
{
  aveassign(s, shared->zone.scrollx, shared->zone.scrolly);
  s->id = ST_BOSSBLOCK;
  s->react = RA_BOSS;
  s->priority = DP_IMPORTANT;
  s->bossSet(P_BIGSQUARE, WHITE, LIGHTGRAY);
  s->dx = 0;
  s->dy = 0;
  if (shared->zone.scrollx == 0)
    s->x = i;
  else
    s->y = i;
}

// make an object and explosion
void SpriteManager::explode(Sprite *s)
{
  speaker.play(random(200));

  s->id = ST_EXPLODE;

  s->x = random((shared->zone.maxright-shared->zone.maxleft)-1)
          +shared->zone.maxleft+1;
  s->y = random((shared->zone.maxbottom-shared->zone.maxtop)-1)
          +shared->zone.maxtop+1;
  s->dx = 0;
  s->dy = 0;
  s->active = ACT_ALIVE;

  for(int a = 0;a < 4;a++)
  {
    if (random(2))
      s->pic[a] = 15;
    else
      s->pic[a] = '*';

    s->scolor[a] = random(16);
  }

  s->bcolor = random(8);

  s->react = RA_NONE;
  s->priority = DP_IMPORTANT;
}  

Sprite* SpriteManager::make(int what, int ax, int ay, int adx, int ady)
{
  Sprite *s2, *s3, *s4, *s = findFreeSprite();
  if (s == NULL)
    return NULL;

  switch(what)
  {
    case ST_SHIPNOSE:
    case ST_SHIPBODY:
    case ST_SHIPFIRE:
      s2 = findFreeSprite();
      s3 = findFreeSprite();
      if (s2 && s3)
        ship3(s, s2, s3, ax, ay);
      break;
    case ST_STAR:
      star(s);
      break;
    case ST_LIGHT:
      light(s, adx, ady, random(8));
      break;
    case ST_COMET:
    case ST_CTAIL1:
    case ST_CTAIL2:
    case ST_CTAIL3:
      s2 = findFreeSprite();
      s3 = findFreeSprite();
      s4 = findFreeSprite();
      if (s2 && s3 && s4)
      comet4(s, s2, s3, s4, adx, ady);
      break;
    case ST_METEOR:
      rock(s, adx, ady);
      break;
    case ST_WARP:
      warp(s, ax, ay, adx, ady);
      break;
    case ST_INSTANT:
      powrup(s, adx, ady, 1, 0, ST_INSTANT);
      break;
    case ST_POWER:
      powrup(s, adx, ady);
      break;
    case ST_WALL:
      wall(s, adx, ady, DARKGRAY, P_BIGSQUARE);
      break;
    case ST_MINE:
      stealth(s, adx, ady);
      break;
    case ST_LASER:
      laser(s, ax, ay);
      break;
    case ST_BOSSLASER:
      s2 = findFreeSprite();
      s3 = findFreeSprite();
      if (s2 && s3)
      bosslaser(s, s2, s3);
      break;
    case ST_BOSSBLOCK:
    case ST_EXPLODE:
      explode(s);
      break;
    case ST_BOMB:
      wall(s, 0, 0, LIGHTGRAY, 4);
      s->id = ST_BOMB;
      break;
    case ST_PCUT:
      wall(s, 0, 0, DARKGRAY, 4);
      s->id = ST_BOMB;
      break;
    default:
    case ST_SPECIAL:
      break;
  }

  return s;
}

void SpriteManager::reset(void)
{
  for (int a = 0;a < numSprites;a++)
    sprite[a].initialize();
}

void SpriteManager::draw(void)
{
  powers = 0;
  instants = 0;

  if (++frame == 4)
    frame = 0;

  safe.clear();

  for (int a = 0;a < numSprites;a++)
  {
    if (sprite[a].priority == DP_BACKGROUND)
      drawHelper(&(sprite[a]));
  } // loop

  shared->player.displayMessage(shared->zone.bcolor);

  for (int a = 0;a < numSprites;a++)
  {
    if (sprite[a].priority == DP_POWER)
    {
      drawHelper(&(sprite[a]));
      if (sprite[a].id == ST_POWER)
	powers++;
      if (sprite[a].id == ST_INSTANT)
        instants++;
      }
  } // loop

  for (int a = 0;a < numSprites;a++)
  {
    if (sprite[a].priority == DP_NORMAL)
      drawHelper(&(sprite[a]));
  } // loop
  for (int a = 0;a < numSprites;a++)
  {
    if (sprite[a].priority == DP_IMPORTANT)
      drawHelper(&(sprite[a]));
    if (sprite[a].id == ST_EXPLODE && sprite[a].active == ACT_ALIVE)
      sprite[a].active = ACT_DYING;
  } // loop
}

void SpriteManager::drawHelper(Sprite *s)
{
    if (s->active == ACT_ALIVE || s->active == ACT_DYING)
    {
      safe.safeErase(s->x, s->y, shared->zone.bcolor);
      s->x += s->dx;
      s->y += s->dy;
    }

    if (s->active == ACT_DYING || s->active == ACT_RESERVED)
      s->active = ACT_DEAD;

    if (s->active > ACT_DEAD)
    {
      if (s->dx != 0 && (s->x <= shared->zone.maxleft-1
            || s->x >= shared->zone.maxright+1))
        s->active = ACT_DEAD;
      else if (s->dy != 0 && (s->y <= shared->zone.maxtop-1
            || s->y >= shared->zone.maxbottom+1))
        s->active = ACT_DEAD; 
      else
      {
        if (s->bcolor != -1)
          safe.safeDraw(s->x, s->y, s->pic[frame], s->scolor[frame],
            s->bcolor);
        else
          safe.safeDraw(s->x, s->y, s->pic[frame], s->scolor[frame],
            shared->zone.bcolor);
      }

      if (s->active == ACT_DEAD)
      {
        switch(s->id)
        {
          case ST_BOMB:
            shared->zone.prob[ST_EXPLODE] += 3;
            break;
        }
      }
    }
}

void SpriteManager::redraw(Sprite *s)
{
  if (s)
  {
    if (s->bcolor != -1)
      cset(s->scolor[frame], s->bcolor);
    else
      cset(s->scolor[frame], shared->zone.bcolor);
    gotoxy(s->x, s->y);
    putch(s->pic[frame]);
  }
}

void SpriteManager::spawn(int px, int py, int pdx, int pdy, int &hits, 
			  int stars)
{
  if (stars)
  {
    Sprite *s;
    for (int a = 0;a < 100;a++)
    {
      if (random(100) < shared->zone.prob[ST_STAR])
      {
        s = findFreeSprite();
        if (s)
          star(s);
      }
    }
    if (shared->zone.special == 3) // boss zone
    {
      int from, to;
      if (shared->zone.scrollx == 0)
      {
        from = shared->zone.maxleft;
        to = shared->zone.maxright;
      }
      else
      {
        from = shared->zone.maxtop;
        to = shared->zone.maxbottom;
      }

      for (int a = from;a <= to;a++)
      {
        s = findFreeSprite();
        if (s)
          bossBlock(s, a);
      }
    }
  }
  else
  {
    // pull on slaves
    for (int a = 0;a < numSprites;a++)
    {
      if (sprite[a].active > ACT_DEAD && sprite[a].slave != NULL)
      {
        sprite[a].slave->active = ACT_RELEASED;
        sprite[a].slave = NULL;
      }
    }
    for (int a = 0;a < numSprites;a++)
    {
      if (sprite[a].active == ACT_RELEASED)
        sprite[a].active = ACT_ALIVE;
    }

    // spawn new things
    int thres;
    int roll;
    for (int times = 0;times < 4;times++)
    {
      thres = 0;
      roll = random(100);
      for (int a = 0;a < NUMSPRITETYPE;a++)
      {
        thres += shared->zone.prob[a];
        if (thres > roll)
        {
          if (a == ST_WALL)
          {
            if (--wallRate <= 0)
            {
              wallRate = random(5) + 3;
              switch(wallType)
              {
                case WT_RANDOM:
                  {// braces keep it from complaning about local variable.
                  for (int b = 0;b < 10;b++)
                    make(ST_WALL, px, py, pdx, pdy);
                  }
                  break;
                case WT_TUNNEL:
                  for (int b = 0;b < NUMHOLES;b++)
                  {
                    
                    holePos[b] += (random(holeSize[b]*2)+1 - holeSize[b]);
                    holeSize[b] += random(3)-1;

                    if (holeSize[b] < 1)
                      holeSize[b] = 1;
                    if (holeSize[b] > 7)
                      holeSize[b] = 7;
                    if (shared->zone.scrollx == 0)
                    {
                      if (holePos[b] < shared->zone.maxleft)
                        holePos[b] = shared->zone.maxleft;
                      if (holePos[b]+holeSize[b] > shared->zone.maxright)
                        holePos[b] = shared->zone.maxright-holeSize[b];
                    }
                    else
                    {
                      if (holePos[b] < shared->zone.maxtop)
                        holePos[b] = shared->zone.maxtop;
                      if (holePos[b]+holeSize[b] > shared->zone.maxbottom)
                        holePos[b] = shared->zone.maxbottom-holeSize[b];
                    }
                  }

                  if (shared->zone.scrollx == 0)
                  {
                    int check;

                    for (int b = shared->zone.maxleft;b <= shared->zone.maxright;b++)
                    {
                      check = 1;
                      for (int c = 0;c < NUMHOLES;c++)
                      {
                        if (b >= holePos[c] && b < holePos[c]+holeSize[c])
                          check = 0;
                      }
                      if (check)
                      {
                        Sprite *s = make(ST_WALL, px, py, pdx, pdy);
                        s->x = b;
                      }
                    }
                  }
                  else
                  {
                    int check;

                    for (int b = shared->zone.maxtop;b <= shared->zone.maxbottom;b++)
                    {
                      check = 1;
                      for (int c = 0;c < NUMHOLES;c++)
                      {
                        if (b >= holePos[c] && b < holePos[c]+holeSize[c])
                          check = 0;
                      }
                      if (check)
                      {
                        Sprite *s = make(ST_WALL, px, py, pdx, pdy);
                        s->y = b;
                      }
                    }
                  }
                  break;
              }   
            }
          }
          else if (shared->zone.special == SL_BOSS &&
	           (a == ST_POWER || a == ST_INSTANT))
          {
            if ((a == ST_POWER && powers < shared->zone.prob[ST_POWER])
	     || (a == ST_INSTANT && instants < shared->zone.prob[ST_INSTANT]))
              make(a, px, py, pdx, pdy);
          }
          else if (a != ST_STAR)
            make(a, px, py, pdx, pdy);

          if (a == ST_EXPLODE)
	  {
            shared->zone.prob[a]--;
	    if (shared->zone.special == SL_BOSS && random(5) != 0)
	      damageBoss(1, hits);
	  }

          a = NUMSPRITETYPE;
        }
      }
    }
  }
}

Sprite* SpriteManager::findFreeSprite(void)
{
  int r = -1;
  for (int a = 0;a < numSprites;a++)
  {
    if (sprite[a].active == ACT_DEAD)
    {
      r = a;
      a = numSprites;
    }
  }

  if (r == -1)
  {
    notfound++;
    return NULL;
  }
  else
  {
    found++;
    sprite[ r ].initialize();
    sprite[ r ].active = ACT_RESERVED;
    return &(sprite[ r ]);
  }
}

void SpriteManager::wallForward(void)
{
  for (int a = 0;a < numSprites;a++)
  {
    if (sprite[a].id == ST_WALL)
    {
      sprite[a].dx = shared->zone.scrollx;
      sprite[a].dy = shared->zone.scrolly;
    }
  }
}

void SpriteManager::wallReset(void)
{
  wallType = random(2);
  for (int a = 0;a < NUMHOLES;a++)
  {
    holePos[a] = random(10);
    holeSize[a] = random(3)+3;
  }
}

int SpriteManager::playerCollide(int x, int y, int &dx, int &dy, int &invinc,
                                 int &effect)
{
  if (invinc > 2)
    return 0;

  int reaction, subgamestat = 0;
  // player collision detection
  for (int a = 0;a < numSprites;a++)
  {
    reaction = 0;
    if (sprite[a].active == ACT_ALIVE)
    {
      if ((x+dx == sprite[a].x && y+dy == sprite[a].y)
       || (x+shared->zone.scrollx+dx == sprite[a].x
        && y+shared->zone.scrolly+dy == sprite[a].y))
       reaction = 1;
      if ((x == sprite[a].x && y == sprite[a].y)
       || (x+shared->zone.scrollx == sprite[a].x
        && y+shared->zone.scrolly == sprite[a].y))
        reaction = 1;
      if (reaction == 1)
      {
        switch (sprite[a].react)
        {
          case RA_BOUNCEHIT:
            sprite[a].bounceme(shared->zone.scrollx+(dx * -1),
                               shared->zone.scrolly+(dy * -1));
          case RA_HARDHIT:
            speaker.play(2000);
            if (invinc > 0)
              ;
            else if (invinc == 0)
            {
              dx = 0;
              dy = 0;

              speaker.play(100);
              delay(100);
              speaker.play(75);
              delay(100);
              speaker.play(50);

              invinc = 1;

              shared->player.items[T_HITS].power
                -= shared->player.items[T_HITS].pUse;
              shared->player.items[T_HITS].soil(ALOT);
	      switch(random(9))
	      {
		case 0:
		case 1:
	        case 2: shared->player.setMessage("Owww!        ", 10); break;
		case 3:
	        case 4: shared->player.setMessage("Doh!         ", 10); break;
	        case 5: shared->player.setMessage("Gotta Hurt   ", 10); break;
	        case 6: shared->player.setMessage("Stings a Bit ", 10); break;
	        case 7: shared->player.setMessage("What Licence?", 10); break;
	        case 8: shared->player.setMessage("Rock? Where? ", 10); break;
              }

              sprite[a].react = 0;
            }
            break;
          case RA_WARP:
            if (shared->player.items[T_HITS].power >= 0 && subgamestat == 0)
	    {
              subgamestat = 4;
	      switch(random(9))
	      {
		case 0:
		case 1:
	        case 2: shared->player.setMessage("Whoops!      ", 10); break;
		case 3:
	        case 4: shared->player.setMessage("Doh!         ", 10); break;
	        case 5: shared->player.setMessage("Start Over   ", 10); break;
	        case 6: shared->player.setMessage("Not Again!   ", 10); break;
	        case 7: shared->player.setMessage("Round & Round", 10); break;
	        case 8: shared->player.setMessage("Warp? Where? ", 10); break;
              }
	    }
            break;
          case RA_POWER:
            speaker.play(3000);
	    if (sprite[a].id == ST_INSTANT)
	    {
	      //shared->player.setMessage("instant      ", 10);
	      effect++;
	    }
	    else
	    {
	      shared->player.plus();
	      shared->player.select().soil(ALOT);
	    }
            sprite[a].active = ACT_DYING;
            break;
          default:
            if (sprite[a].react >= RA_POWER)
            {
              speaker.play(3000);
              shared->player.plus(sprite[a].react-RA_POWER, 1);
              sprite[a].active = ACT_DYING;
            }
            break;
        } // end switch
      }
    }
  }// end player collision detection
  return subgamestat;
}

void SpriteManager::objectCollide(int &hits)
{
  // object collision detection
  for (int c = 0;c < numSprites;c++)
  {
    if (sprite[c].active == ACT_ALIVE && sprite[c].react == RA_FORCER
     || sprite[c].react == RA_BOSS)
    {
      for (int a = 0;a < numSprites;a++)
      {
        if (sprite[a].active == ACT_ALIVE)
        {
          if ( (sprite[c].x == sprite[a].x
             || sprite[a].x+sprite[a].dx == sprite[c].x)
            && (sprite[c].y == sprite[a].y
             || sprite[a].y+sprite[a].dy == sprite[c].y) )
            ocHelper(a, c, hits);
        } // end is it active
      } // end big loop
    } // end is it a bouncer
  } // end 'laser' loop

  if (bossHit == 5)
  {
    char bob[15];
    switch(random(9))
    {
      case 0:
      case 1:
      case 2: 
	sprintf(bob,                    "Only %3d left", hits);
        shared->player.setMessage(bob, 10); break;
      case 3:
      case 4: shared->player.setMessage("That's It!   ", 10); break;
      case 5: 
      case 6: shared->player.setMessage("Go! Go! Go!  ", 10); break;
      case 7: shared->player.setMessage("You Can Do It", 10); break;
      case 8: shared->player.setMessage("Whoop It!    ", 10); break;
    }
  } 
}// end fuction

void SpriteManager::ocHelper(int a, int c, int &hits)
{
  if (sprite[a].react == RA_FORCER)
    ;
  else if (sprite[c].react == RA_BOSS)
  {
    if (sprite[a].id == ST_BOSSLASER)
    {
      damageBoss(1, hits);
    }
  }
  else if (sprite[a].react == RA_BOUNCEHIT)
  {
    speaker.play(2000);
    sprite[a].react = 0;
    sprite[a].bounceme(sprite[c].dx * -1, sprite[c].dy * -1);
    sprite[c].bounceme(sprite[c].dx, sprite[c].dy);
  }
  else if (sprite[a].react != RA_NONE && sprite[a].react != RA_BOSS)
  {
    sprite[c].bounceme(sprite[c].dx, sprite[c].dy);
    speaker.play(2000);
    if (sprite[a].id == ST_WALL)
    {
      for (int w = 0;w < numSprites;w++)
      {
        if (sprite[w].id == ST_WALL
         && ( (shared->zone.scrollx == 0
          && sprite[a].y == sprite[w].y)
         || (shared->zone.scrolly == 0
          && sprite[a].x == sprite[w].x) ) )
        {
          sprite[w].dx = shared->zone.scrollx * -1;
          sprite[w].dy = shared->zone.scrolly * -1;
        }
      } // end wall loop
    } // end if it's a wall
  } // end if something else
} 

void SpriteManager::reflectAll(ItemType *i)
{
  for (int a = 0;a < numSprites;a++)
  {
    if (sprite[a].active == ACT_ALIVE && sprite[a].react != 0
     && !(sprite[a].id >= ST_COMET && sprite[a].id <= ST_CTAIL3)
     && sprite[a].id != ST_BOSSBLOCK)
    {
      if (i == NULL || i->power >= i->pUse)
      {
        if (i != NULL)
          i->power -= i->pUse;
        sprite[a].bounceme(sprite[a].dx, sprite[a].dy);
        if(sprite[a].id != ST_POWER)
          sprite[a].react = 0;
      }
    }
  }
}

void SpriteManager::powerAll(void)
{
  for (int a = 0;a < numSprites;a++)
  {
    if (sprite[a].active == ACT_ALIVE // && item.power >= item.pUse
     && (sprite[a].id == ST_METEOR || sprite[a].id == ST_WALL
      || sprite[a].id == ST_MINE) )
    {
      //item.power -= item.pUse;
      powrup(&(sprite[a]), shared->zone.scrollx, shared->zone.scrolly, 0);
    }
  }
}

void SpriteManager::damageBoss(int howMuch, int &hits)
{
  hits -= howMuch;
  speaker.play(hits+32);
  bossHit = 0;
}

void SpriteManager::bossSet(int hits)
{
  int bp, bfc, bbc;

    if (hits > 550) bp = 219;
    else if (hits > 500) bp = 178;
    else if (hits > 450) bp = 177;
    else if (hits > 400) bp = 176;
    else if (hits > 350) bp = 32;
    else if (hits > 300) bp = 176;
    else if (hits > 250) bp = 177;
    else if (hits > 200) bp = 178;
    else if (hits > 150) bp = 219;
    else if (hits > 100) bp = 178;
    else if (hits > 50) bp = 177;
    else if (hits > 0) bp = 176;

    if (bossHit == 0)
    {
      bfc = YELLOW;
      bbc = RED;
    }
    else if (hits > 400)
    {
      bfc = WHITE;
      bbc = LIGHTGRAY;
    }
    else if (hits > 200)
    {
      bfc = DARKGRAY;
      bbc = LIGHTGRAY;
    }
    else if (hits > 0)
    {
      bfc = DARKGRAY;
      bbc = BLACK;
    }

    for (int a = 0;a < numSprites;a++)
      if (sprite[a].id == ST_BOSSBLOCK)
        sprite[a].bossSet(bp, bfc, bbc);

  if (bossHit < 100)
  {
    bossHit++;
    if (bonusOn == 1)
      shared->zone.prob[ST_POWER]--;
    bonusOn = 0;
  }
  else if (bonusOn == 0)
  {
    shared->zone.prob[ST_POWER]++;
    bonusOn = 1;
  }

}

void SpriteManager::killBoss(void)
{
  Sprite *s;

  for (int a = 0;a < numSprites;a++)
  {
    if (sprite[a].id != ST_SHIPNOSE && sprite[a].id != ST_SHIPBODY
     && sprite[a].id != ST_SHIPFIRE)
      sprite[a].active = 0;
  }
  for (int a = 0;a < 25;a++)
  {
    s = findFreeSprite();
    if (s)
    {
      powrup(s, shared->zone.scrollx, shared->zone.scrolly, 1, T_FUEL);
      s->dy = 0;
      s->dx = 0;
    }
  }
  shared->player.setMessage("SEE-YA!     ", 20);
}

void SpriteManager::catchPower(void)
{
  for (int a = 0;a < numSprites;a++)
  {
    if (sprite[a].active == ACT_ALIVE && sprite[a].react >= RA_POWER)
    {
      switch(shared->zone.scrollx + shared->zone.scrolly * 2)
      {
        case 1: // scrolling right
          if (sprite[a].x + sprite[a].dx == shared->zone.maxright - 8)
            sprite[a].dx = 0;
          break;
        case -1: // scrolling left
          if (sprite[a].x + sprite[a].dx == shared->zone.maxleft + 8)
            sprite[a].dx = 0;
          break;
        case 2: // scrolling down
          if (sprite[a].y + sprite[a].dy == shared->zone.maxbottom - 8)
            sprite[a].dy = 0;
          break;
        case -2: // scrolling up
          if (sprite[a].y + sprite[a].dy == shared->zone.maxtop + 8)
            sprite[a].dy = 0;
          break;
      }
    }
  }
}

void SpriteManager::home(int what, int x, int y)
{
  for (int a = 0;a < numSprites;a++)
  {
    if (sprite[a].active == 1 && sprite[a].id == what)
    {
     
      if (shared->zone.scrollx == 0)
      {
        sprite[a].dx = sign(x - sprite[a].x);
        if (sprite[a].dx == 0)
          sprite[a].dy = sign(y - sprite[a].y);
      }
      else
      {
        sprite[a].dy = sign(y - sprite[a].y);
        if (sprite[a].dy == 0)
          sprite[a].dx = sign(x - sprite[a].x);
      }
    }
  }
}

void SpriteManager::spreadEffect(int px, int py, int which)
{
  Sprite *s;

  for (int dx = -2;dx <= 2;dx++)
   {
    for (int dy = -2;dy <= 2;dy++)
    {
      if (dx == 0 && dy == 0)
        continue;
      s = make(which, px, py, 0, 0);
      if (s)
      {
        s->x = px;
        s->y = py;
        s->dx = dx;
        s->dy = dy;
        s->react = RA_NONE;
        if (s->id != ST_BOMB)
          s->id = ST_STAR;
      }
    }
  }
}

void SpriteManager::pingEffect(void)
{
  for (int a = 0;a < numSprites;a++)
  {
    if (sprite[a].active == ACT_ALIVE
     && sprite[a].id == ST_MINE)
    {
      for (int b = 0;b < 4;b++)
      {
        if (sprite[a].scolor[b] == shared->zone.bcolor)
       	  sprite[a].scolor[b] = LIGHTGREEN;
      }
    }
  }
}
