// sprite class for the ASTER@IDS game 
#ifndef ASPRITE_H
#define ASPRITE_H

class Sprite;
class SpriteManager;

#include "const.h"
#include "ashared.h" // shared def
#include "buffer.h"

class Sprite
{
public:
  void initialize(void);
  void bounceme(int bdx, int bdy);

  void flashTo(int c)
    {scolor[1] = c; scolor[3] = c;}
  void reColor(int c)
    {scolor[0] = c; scolor[1] = c; scolor[2] = c; scolor[3] = c;}
  void bossSet(int bp, int bfc, int bbc);

  int id, active, react;
  int x,y,dx,dy;
  int pic[4], scolor[4], bcolor;
  int priority;

  Sprite *slave;
};

class SpriteManager
{
public:
  SpriteManager(int n);

  void initialize(Shared *s)
    {shared = s; bonusOn = 0;}

  Sprite* findFreeSprite(void);

  void aveassign(Sprite *b, int bdx, int bdy);

  void ship3(Sprite *s1, Sprite *s2, Sprite *s3, int ax, int ay);
  void powrup(Sprite *s, int adx, int ady, int all = 1, int special = 0, 
              int force = 0);
  void rock(Sprite *s, int adx, int ady);
  void comet4(Sprite *h, Sprite *t1, Sprite *t2, Sprite *t3, int adx, int ady);
  void star(Sprite *s);
  void wall(Sprite *s, int adx, int ady, int wcolor, int wpic);
  void laser(Sprite *s, int ax, int ay);
  void fwall(Sprite *s, int ax, int ay);
  void light(Sprite *s, int adx, int ady, int lcolor);
  void warp(Sprite *s, int ax, int ay, int adx, int ady);
  void stealth(Sprite *s, int adx, int ady);
  void bosslaser(Sprite *s1, Sprite *s2, Sprite *s3);
  void bossBlock(Sprite *s, int i);
  void explode(Sprite *s);
  void bomb(Sprite *s, int ax, int ay, int adx, int ady);

  Sprite* make(int what, int ax, int ay, int adx, int ady);

  void reset(void);

  void draw(void);
  void redraw(Sprite *s);
  void spawn(int px, int py, int pdx, int pdy, int &hits, int stars = 0);

  void wallForward(void);
  void wallReset(void);

  // return is new subgamestat
  int playerCollide(int x, int y, int &dx, int &dy, int &invinc, int &effect);

  void objectCollide(int &hits);
  void ocHelper(int a, int c, int &hits);

  void reflectAll(ItemType *i);
  void powerAll(void);

  void damageBoss(int howMuch, int &hits);
  void bossSet(int hits);
  void killBoss(void);
  void catchPower(void);
  void home(int what, int x, int y);

  void spreadEffect(int px, int py, int which);
  void pingEffect(void);

  void reportStats(void)
  {
    gotoxy(1,1);
    cset(LIGHTGRAY);
    cprintf("rebuilds: %d\r\n", rebuilds);
    cprintf("found: %d\r\n", found);
    cprintf("not found: %d\r\n", notfound);
  }

protected:
  Buffer safe;
  Sprite *sprite;
  Shared *shared;
  int rebuilds, found, notfound;
  int numSprites;
  int frame;
  int wallRate, wallType, holePos[NUMHOLES], holeSize[NUMHOLES];
  int bossHit, bonusOn, powers, instants;

  void drawHelper(Sprite *s);
};

#endif
