//the ASTER@IDS game and related functions

#include <conio.h> // color constants
#include "gamefunc.h" // difficulty
#include "azone.h" // class def
#include "mapfunc.h" // zone constants

enum speed_type {SP_SLOWEST = 9, SP_SLOW = 11, SP_SLOWER = 13,
  SP_MEDIUM = 14, SP_FASTER = 15, SP_FAST = 16, SP_FASTEST = 17};

//shortcut for define()
void Zone::sc(int sp, int m, int s, int l, int c, int w, int p, int b, int t)
{
  switch(difficulty)
  {
    case D_EASY:
      speed = sp * 0.8;
      instantProb = 1;
      break;
    case D_MEDIUM:
      speed = sp;
      instantProb = 10;
      break;
    case D_HARD:
      speed = sp * 1.2;
      instantProb = 10;
      break;
    case D_RIDICULOUS:
      speed = sp * 1.4;
      instantProb = 10;
      break;
  }

  for (int a = 0;a < NUMSPRITETYPE;a++)
    prob[a] = 0;

  prob[ST_METEOR] = m;
  prob[ST_STAR] = s;
  prob[ST_LIGHT] = l;
  prob[ST_COMET] = c;
  prob[ST_WARP] = w;
  prob[ST_POWER] = p;
  prob[ST_WALL] = b;
  prob[ST_MINE] = t;
}

// definintions for zone types on map
void Zone::define(int type)
{
  maxtop = 2;
  maxleft = 2;
  maxbottom = 24;
  maxright = 24;
  special = 0;
  bcolor = 0;
  switch (type)
  {                   // sp          met str lgt com wrp pow wal mine
    case Z_EMPTY:     sc(SP_SLOWEST, 2,  20, 0,  0,  1,  0,  0,  0); break;
    case Z_SLOWROCK:  sc(SP_SLOW,    10, 22, 0,  0,  1,  1,  0,  0); break;
    case Z_MEDROCK:   sc(SP_SLOWER,  20, 10, 0,  1,  4,  2,  0,  0); break;
    case Z_FASTROCK:  sc(SP_FAST,    27, 6,  0,  0,  5,  2,  0,  0); break;
    case Z_SLOWCOMET: sc(SP_SLOW,    4,  10, 0,  10, 3,  1,  0,  0); break;
    case Z_MEDCOMET:  sc(SP_MEDIUM,  13, 10, 0,  15, 3,  2,  0,  0); break;
    case Z_FASTCOMET: sc(SP_FASTEST, 0,  6,  0,  30, 1,  3,  0,  0); break;
    case Z_SLOWWARP:  sc(SP_SLOW,    0,  24, 0,  0,  10, 1,  0,  0); break;
    case Z_MEDWARP:   sc(SP_SLOWER,  10, 12, 0,  0,  15, 2,  0,  0); break;
    case Z_FASTWARP:  sc(SP_FASTER,  10, 12, 0,  0,  23, 3,  0,  0); break;
    case Z_SLOWWALL:  sc(SP_SLOW,    0,  0,  20, 0,  0,  1,  10, 0);
       bcolor = BROWN;
       break;
    case Z_MEDWALL:   sc(SP_MEDIUM,  0,  0,  10, 0,  0,  2,  15, 0);
       bcolor = CYAN;
       break;
    case Z_FASTWALL:  sc(SP_FAST,    0,  0,  10, 0,  0,  3,  15, 0);
       bcolor = CYAN;
       break;
    case Z_SLOWMINE:  sc(SP_SLOWEST, 5,  22, 0,  0,  0,  2,  0,  5); break;
    case Z_MEDMINE:   sc(SP_SLOWEST, 0,  18, 0,  0,  0,  3,  0,  15); break;
    case Z_FASTMINE:  sc(SP_SLOW,    0,  0,  7,  0,  0,  4,  0,  23);
       bcolor = CYAN;
       break;
    case Z_DISTROCK:  sc(SP_SLOWEST, 10, 22, 0,  0,  0,  6,  0,  0);
       special = SL_DISRUPT;
       break;
    case Z_DISTCOMET: sc(SP_SLOW,    0,  6,  0,  10, 0,  8,  0,  0);
       special = SL_DISRUPT;
       break;
    case Z_DISTWARP:  sc(SP_SLOWEST, 0,  20, 0,  0,  5,  6,  0,  0);
       special = SL_DISRUPT;
       break;
    case Z_TOP:       sc(SP_SLOWER,  15, 14, 0,  0,  4,  1,  0,  0);
       maxbottom = 14;
       break;
    case Z_RIGHT:     sc(SP_SLOWER,  15, 14, 0,  0,  4,  1,  0,  0);
       maxleft = 12;
       break;
    case Z_LEFT:      sc(SP_SLOWER,  15, 14, 0,  0,  4,  1,  0,  0);
       maxright = 14;
       break;
    case Z_BOTTOM:    sc(SP_SLOWER,  15, 14, 0,  0,  4,  1,  0,  0);
       maxtop = 12;
       break;
    case Z_SLOWHUNK:  sc(SP_SLOW,    5,  20, 0,  0,  0,  1,  0,  0); break;
    case Z_MEDHUNK:   sc(SP_SLOW,    10, 21, 0,  0,  1,  1,  0,  0); break;
    case Z_FASTHUNK:  sc(SP_SLOWER,  15, 14, 0,  0,  4,  2,  0,  0); break;
    default: 
    case Z_SOLID:     sc(0,          0,  0,  0,  0,  0,  0,  0,  0); break;
    case Z_BOSS:      sc(SP_MEDIUM,  0,  0,  0,  0,  1,  0,  0,  0); // boss
       special = SL_BOSS;
       prob[ST_BOSSLASER] = 20; //37
       prob[ST_INSTANT] = 1;
       break;
    case Z_SECRET:     sc(SP_MEDIUM, 0,  0,  17, 0,  0,  10, 0,  0); 
       // secret area
       special = SL_SECRET;
       bcolor = CYAN;
       if (scrollx != 0)
       {
         maxtop += 5;
         maxbottom -= 5;

       }
       if (scrolly != 0)
       {
         maxleft += 5;
         maxright -= 5;
       }
       break;
  }

  if (type >= Z_SLOWHUNK && type <= Z_FASTHUNK)
  {
    if (scrolly == -1)
      maxright = 14;
    else if (scrolly == 1)
      maxleft = 12;
    else if (scrollx == -1)
      maxtop = 12;
    else if (scrollx == 1)
      maxbottom = 14;
  }
}
