// the DODGEBEAM game and related fucntions

#include "gamefunc.h"
#include "escfp23x.h"
#include "db.h"

enum stufftype {ST_PLAYER, ST_PRIZE, ST_BARRIER};
const char SMILEY = 2;
const char STAR = 15;
const char TRIPHBAR = 240;
const char DOUBHBAR = 205;
const char SMALLBOX = 254;
const char LARGEBOX = 219;

void DBItem::set(int w, int ypos, char p, int c)
{
  active = 1;
  what = w;
  y = ypos;
  x = 5;
  pic = p;
  color = c;

  drawnX = 1;
  drawnY = 1;
  drawn = 0;
  changed = 1;
}

void DBItem::draw(void)
{
  cset(color, BLACK);
  gotoxy(x, y);
  putch(pic);

  drawnX = x;
  drawnY = y;
  drawn = 1;
  changed = 0;
}

void DBItem::erase(void)
{
  if (!drawn)
    return;
  cset(BLACK);
  gotoxy(drawnX, drawnY);
  putch(SPACE);

  drawn = 0;
}

void DBItem::move(void)
{
  if (changed)
    erase();  // erase checks against drawn too

  if (!drawn)
    draw();
}

// C++ version of my dodgbeam game

void DodgeBeam::initialize(void)
{
  for (int a = 0;a < NUMSTUFF;a++)
    stuff[a].kill(0);

  lives = 3;
  stuff[0].set(ST_PLAYER, 10, SMILEY, YELLOW);
  for (int a = 0;a < NUMLASER;a++)
    laser[a] = 1;
  lasernum = NUMLASER;

  gamestat = 0;
  battery = 0;

  laserColor = 0;
  length = 1;
  laserY = 0;
  status = 0;
  shuffleTime = 0;

  pull = -1;

  death = 0;

  setDifficulty();
}

int DodgeBeam::game(void)
{
  int x, y, dy;

  initialize();

  // status = 1 if firing, 2 if retracting, and 0 if niether

  cset(WHITE);
  clrscr();

  drawFrame();

  readysetgo(gamestat, (int)'$');

  frameControl.set(frameRate);
  frameControl.sync();
  do
  {
    // character movement code
    dy = 0;
    if (death == 0 && pull != 0)
    {
      switch (getkey())
      {
        case UP: dy = -1; break;
        case DOWN: dy = 1; break;
        case LEFT:
          destroyBarriers();
          break;
        case RIGHT:
          fireLaser();
          break;
        case ESCAPE: pause(gamestat); break;
        case F2:
          toggleLaserSound();
          break;
      }

      y = stuff[0].getY();
      x = stuff[0].getX();

      if ((y+dy >= 12) || (y+dy <= 1))
      {
        dy = 0;
        speaker.play(100);
      }

      for (int a = 1;a < NUMSTUFF;a++)
      {
        if ( stuff[a].isActive()
         && y+dy == stuff[a].getY()
         && x == stuff[a].getX() )
        {
          switch( stuff[a].isWhat() )
          {
            case ST_PRIZE:
              stuff[a].kill();
              battery++;
              updateBat();
              if (battery >= 10)
                speaker.play(70*battery);
              else
                speaker.play(40*battery);
              break;
            case ST_BARRIER:
              dy = 0;
              speaker.play(100);
              break;
          }
        }
      }
      // update the player
      stuff[0].setY( stuff[0].getY()+dy );
                     
    }  // end if not dead
    frameControl.force();
    speaker.stop();

    // laser and object code
  
    // shrank back to cannon
    if (length == 1 && status == 2 && pull != -1)
    {
      if (stuff[pull].isWhat() != ST_PLAYER)
        stuff[pull].kill();
      pull = -1;
    }
    if ((length == 0) && (status == 2))
    {
      status = 3;
      shuffleTime = 13;
 
      gotoxy(39,laserY);
      putch(SPACE);

      // check for death
      if (death == 1)
      {
        loseLife();
        if (lives < 1)
          gamestat = 1;
      }
    }
    // see if maxxed out
    if ((40-(length-1)) == 4)
    {
      status = 2;
      pull = -1;
    }
    // check to see if hit stuff
    if (status == 1)
    {
      for (int a = 0;a < NUMSTUFF;a++)
      {
        if ( stuff[a].isActive()
          && stuff[a].getX() == 40-(length-1)
          && stuff[a].getY() == laserY )
        {
          status = 2;
          pull = a;

          switch(stuff[a].isWhat())
          {
            case ST_PLAYER:
              lives--;
              death = 1;
              break;
            case ST_PRIZE:
              battery--;
              updateBat();
              break;
          }
        } // end inner if
      } // end for
    } // end if status == 1

    // manage status
    switch (status)
    {
      case 0:
        shuffle();
        break;
      case 1: length++; break;
      case 2: length--; break;
      case 3:
        if (difficulty != D_EASY)
	  shuffleCannons();
	else
	  status = 0;
        break;
    }

    if (laserSound)
      speaker.play((length*10));
    else if (status == 1 && length < 4)
      speaker.play(1000 - length*10);

    // draw stuff

    // draw the laser
    gotoxy(40-length, laserY);

    cset(laserColor++);
    if (laserColor == 16) laserColor = 1;
    for (int a = 0; a < length; ++a)
      putch(DOUBHBAR);
    // maybe drag something along
    if (status == 2 && pull != -1)
    {
      stuff[pull].setX(40-length);
    }
    gotoxy(40-(length+1), laserY);
    putch(SPACE);

    // make sure item display isn't corrupted.
    if (length == 30 && status == 2)
    {
      for (int a = 0;a < NUMSTUFF;a++)
        stuff[a].refresh();
    }

    // draw everything else.
    for (int a = 0;a < NUMSTUFF;a++)
    {
      if (stuff[a].isActive())
      {
        stuff[a].move();
      }
    }

  // end laser and object code

  } while (gamestat == 0);
  speaker.stop();

  // pre-end
  if (gamestat == 2)
  {
    speaker.stop();
    gotoxy(6,laserY);
    cputs("                                  ");
    for (int a = 2;a < 12;a++)
    {
      gotoxy(5, a);
      putch(' ');
    }
    cset(YELLOW);
    for (int a = 0;a < 5;a++)
    {
      gotoxy(5, y);
      putch(' ');
      y += sign(6 - y);
      gotoxy(5, y);
      putch(2);
      delay(1000);
    }
  }
  if (gamestat == 3 || gamestat == 1)
    lives = 0;
  return gamestat;
}

void DodgeBeam::endSequence(void)
{
  int a, b;
  delay(2000);
  //enter the cannon
  cset(LIGHTGRAY);
  gotoxy(40, 2);
  putch(220);
  gotoxy(40, 3);
  putch(' ');
  gotoxy(40, 4);
  putch(223);

  speaker.play(100);
  delay(250);
  speaker.stop();
  delay(250);

  gotoxy(39, 2);
  putch(220);
  putch(219);
  gotoxy(29, 3);
  cputs("  ");
  gotoxy(39, 4);
  putch(223);
  putch(219);

  speaker.play(100);
  delay(250);
  speaker.stop();
  delay(250);
  //the wall
  for (a = 3;a < 12;a++)
  {
    gotoxy(40, a);
    putch(219);
  }
  drawEndCannon(2);
  //aiming...
  for (int a = 3;a < 7;a++)
  {
    gotoxy(38, a-1);
    cputs("  ");
    drawEndCannon(a);
  }
  beepbeep(1000, 25, 10);
  delay(100);
  drawEndCannon(6);
  drawEndCannon(5);
  drawEndCannon(4);
  beepbeep(1000, 25, 10);
  delay(100);
  gotoxy(38,4);
  cputs("  ");
  drawEndCannon(5);
  beepbeep(1000, 25, 10);
  delay(1000);
  // power up...
  for (int a = 500;a < 1000;a += 8)
  {
    speaker.play(a);
    cset(BLACK, LIGHTGRAY);
    gotoxy(40,6);
    putch(254);
    delay(50);
    speaker.play(a+4);
    cset(RED, LIGHTGRAY);
    gotoxy(40,6);
    putch(254);
    delay(50);
  }
  //step to the side...
  cset(YELLOW);
  gotoxy(5,6);
  putch(' ');
  gotoxy(5,5);
  putch(2);
  delay(500);
  //and a miss!
  for (a = 0;a < 40;a++)
  {
    speaker.play(100-a*2);
    b = random(a);
    if (b == 0 || b == 10 || b == 20 || b == 30) cset(random(16));
    else cset(WHITE);
    gotoxy(2, 6);
    for (b = 2;b < 40;b++) putch(219);
    delay(50);
  }
  cset(WHITE);
  gotoxy(2, 6);
  for (b = 2;b < 40;b++) putch(219);
  //there's the exit!
  gotoxy(1,6);
  cset(LIGHTBLUE);
  putch(219);
  delay(200);
  gotoxy(1,6);
  cset(RED);
  putch(219);
  delay(200);
  gotoxy(1,6);
  cset(LIGHTRED);
  putch(219);
  delay(200);
  gotoxy(1,6);
  cset(WHITE);
  putch(219);
  delay(200);
  gotoxy(1,6);
  cset(WHITE);
  putch(178);
  delay(200);
  gotoxy(1,6);
  cset(WHITE);
  putch(177);
  delay(200);
  gotoxy(1,6);
  cset(WHITE);
  putch(176);
  delay(200);

  speaker.stop();
  gotoxy(1,6);
  cputs("                                       ");
  //freedom.
  delay(500);
  cset(YELLOW);
  gotoxy(5,5);
  putch(' ');
  gotoxy(5,6);
  putch(2);
  delay(500);
  gotoxy(4,6);
  putch(2);
  putch(' ');
  delay(500);
  gotoxy(3,6);
  putch(2);
  putch(' ');
  delay(500);
  gotoxy(2,6);
  putch(2);
  putch(' ');
  delay(500);
  gotoxy(1,6);
  putch(2);
  putch(' ');
  delay(500);
  gotoxy(1,6);
  putch(' ');
  delay(700);
} // end endSequence

void DodgeBeam::instructions(void)
{
  while (getkey())
    ;
  titlebar("game A: DODG#B#AM", 11, 11);
  cset(LIGHTCYAN);
  //     0000000001111111111222222222233333333334
  //     1234567890123456789012345678901234567890
  cputs("To see if humans are a species worhty of");
  cputs("interplanitary respect, the aliens put  ");
  cputs("you in a chamber to determine your, and ");
  cputs("Earth's future.  To prove that you are  ");
  cputs("worth release, you must destroy all of  ");
  cputs("the laser cannons in the room.  To do   ");
  cputs("this you must collect charges for your  ");
  cputs("laser gun until the arrow turns red.  If");
  cputs("you fire before this you will lose all  ");
  cputs("your charges and have to collect them   ");
  cputs("agian.  If you're caught in a tight spot");
  cputs("and a laser is coming, you can destroy  ");
  cputs("nearby barriers, this costs 2 charges.  \r\n");

  cputs("--The small red boxes are your charges. ");
  cputs("--The large blue boxes are barriers.    ");
  cputs("--The lasers can take the barriers away.");
  cputs("--They can take, and subtract charges.  ");
  cputs("--Beware!  Lasers can take your lives!  \r\n");

  cputs("--Up and down arrows move up and down.  ");
  cputs("--Right arrow fires your laser.         ");
  cputs("--Left arrow destroys adjacent barriers.");
  cset(GREEN);
  cputs("       --PRESS A KEY TO CONTINUE--     ");
  getch();
}

void DodgeBeam::drawFrame(void)
{
  gotoxy(1,1);
  cset(BLUE);
  for (int b = 1; b <= 40; b++)
    putch(LARGEBOX);

  gotoxy(1,12);
  for (int c = 1; c <= 40; c++)
    putch(LARGEBOX);

  for (int c = 2; c <= 12; c++)
  {
    gotoxy(1,c);
    putch(LARGEBOX);
  }

  cset(BLACK, BLUE);
  gotoxy(18,12);
  putch(25);
  cset(WHITE);

  drawCannons();

  gotoxy(1,14);
  cputs("LIVES:");
  cset(YELLOW);
  putch(1); putch(1); putch(2);
  gotoxy(1,13);
  cset(DARKGRAY, LIGHTGRAY);
  cputs("BATTERY:");
  cset(BROWN, LIGHTGRAY);
  for (int d = 0; d < maxBattery; d++)
    putch(16);
}

// shows the battery's power bar in dodgebeam
void DodgeBeam::updateBat(int erase)
{
  if (battery > maxBattery)
    battery = maxBattery;
  if (battery < 0)
    battery = 0;

  int k;
  if (battery > 0 && battery <= maxBattery)
  {
    gotoxy(8+battery,13);
    cset(YELLOW, LIGHTGRAY);
    putch(16);
  }
  if (battery >= 0 && battery < maxBattery)
  {
    gotoxy(9+battery,13);
    cset(BROWN, LIGHTGRAY);
    if (erase == 10)
      erase = maxBattery-battery;
    for (k = 0;k < erase;k++)
      putch(16);
  }
  gotoxy(18, 12);
  if (battery >= 10) cset(RED, BLUE);
  else cset(BLACK, BLUE);
  putch(25);
  cset(LIGHTGRAY);
}

void DodgeBeam::drawCannons(void)
{
  for (int a = 0;a < NUMLASER;a++)
  {
    gotoxy(40, a+2);
    if (laser[a])
      cset(WHITE);
    else
      cset(DARKGRAY);
    putch(TRIPHBAR);
  }
}      

void DodgeBeam::destroyBarriers(void)
{
  int y = stuff[0].getY();

  if (battery >= 2)
  {
    for (int a = 1;a < NUMSTUFF;a++)
    {
      if (stuff[a].isActive() && stuff[a].isWhat() == ST_BARRIER
       && stuff[a].getX() == stuff[0].getX()
       && (stuff[a].getY()+1 == y
        || stuff[a].getY()-1 == y) )
        stuff[a].kill();
    }

    battery -= 2;
    updateBat(2);
    speaker.play(100);
    cset(WHITE, RED);
    gotoxy(5, y+1);
    putch(42);
    gotoxy(5, y-1);
    putch(42);
    for (int d = 0; d < 10; d++)
    {
      delay(50);
      speaker.play(random(100));
    }
    cset(BLUE);
    gotoxy(5, y+1);
    putch(32);
    gotoxy(5, y-1);
    putch(32);
    gotoxy(5,1);
    putch(219);
    gotoxy(5,12);
    putch(219);
    speaker.stop();
  }
}

void DodgeBeam::fireLaser(void)
{
  int firemax, firing;
  int y = stuff[0].getY();
  int hit = 0;

  if (battery >= 10)
    firemax = 34;
  else
    firemax = battery * 3;
  firing = 0;
  do
  {
    cset(random(15)+1);
    gotoxy(6, y);
    for (int b = 0;b < firing;b++)
    putch(196);
    speaker.play(firing * 5);
    delay (30);
  } while (firing++ < firemax);
  if (battery >= 10)
  {
    battery -= 10;
    putch(15);

    updateBat(10);
    if (laser[y - 2] == 1)
    {
      hit = 1;
      laser[y - 2] = 0;
      lasernum--;
      setDifficulty();
    }

    speaker.playFor(100, 1000);
    delay(500);

    if (hit)
    {
      for (int a = 1000;a < 2000;a+=100)
        speaker.playFor(a, 50);
    }
    else
      speaker.playFor(50, 1000);

    if (lasernum == 0)
      gamestat = 2;
    gotoxy(6, y);
    for (int b = 0;b < 34; b++)
      putch(32);
  }
  else
  {
    battery = 0;
    updateBat(10);
    gotoxy(6, y);
    for (int b = 0;b < firemax; b++)
      putch(32);
  }

  drawCannons();
}

void DodgeBeam::shuffle(void)
{

  length = 0;
  status = 1;

  drawCannons();

  // choose a laser to fire
  do
  {
    laserY = random(NUMLASER)+2;
  } while ((laser[laserY - 2] == 0) && (lasernum > 0));

  pull = -1;

  // put a prize, but not on the player.
  if (stuff[1].isActive() == 0)
  {
    stuff[1].set(ST_PRIZE, 0, SMALLBOX, RED);
    do
    {
      stuff[1].setY( random(10)+2 );
    } while (stuff[1].getY() == stuff[0].getY());
  }

  for (int e = 0;e < totalbar;e++)
  {
    stuff[e+2].kill();
    do
    {
      stuff[e+2].set(ST_BARRIER, random(10)+2, LARGEBOX, BLUE);
    } while ( stuff[e+2].getY() == stuff[0].getY()
           || stuff[e+2].getY() == stuff[1].getY() );
  }
}

void DodgeBeam::loseLife(void)
{
  death = 0;
  for (int e = 50;e < 500;e += 50)
  {
    speaker.play(e);
    gotoxy(7+lives,14);
    cset(RED);
    putch(88);
    delay(75);

    speaker.stop();
    gotoxy(7+lives,14);
    cset(YELLOW);
    putch(2);
    delay(75);
  }
  if (lives > 0)
  {
    gotoxy(6+lives,14);
    cset(YELLOW);
    putch(2);
  }
  gotoxy(7+lives,14);
  putch(32);

  stuff[0].setX(5);
}

void DodgeBeam::shuffleCannons(void)
{
  if (--shuffleTime < 0)
    status = 0;

  if (shuffleTime % 4 == 0)
    speaker.play(100);

  if (shuffleTime % 2 == 0)
    return;

  int tryLaser;
  int toDistribute = lasernum;

  for (int a = 0;a < NUMLASER;a++)
    laser[a] = 0;
  while (toDistribute > 0)
  {
    tryLaser = random(NUMLASER);
    if (laser[tryLaser] == 0)
    {
      laser[tryLaser] = 1;
      toDistribute--;
    }
  }
  drawCannons();
}


//draws the super cannon in DODGEBEAM finale
void DodgeBeam::drawEndCannon(int cy, int cx)
{
  gotoxy(cx, cy+3);
  cputs("  ");
  gotoxy(cx, cy);
  putch(220);
  putch(219);
  putch(219);
  gotoxy(cx, cy+1);
  cputs("  ");
  cset(BLACK, LIGHTGRAY);
  putch(254);
  cset(LIGHTGRAY);
  gotoxy(cx, cy+2);
  putch(223);
  putch(219);
  putch(219);

  speaker.play(100);
  delay(250);
  speaker.stop();
  delay(250);
}

void DodgeBeam::setDifficulty(void)
{
  switch(difficulty)
  {
    case D_EASY:
      maxBattery = 30;
      frameRate = 40 - (lasernum * 2);
      totalbar = (10.0-lasernum) * 0.3;
      break;
    case D_MEDIUM:
      maxBattery = 20;
      frameRate = 50 - (lasernum * 2.5);
      totalbar = (10.0 - lasernum) * 0.5;
      break;
    case D_HARD:
      maxBattery = 13;
      frameRate = 60 - (lasernum * 3);
      totalbar = (10.0 - lasernum) * 0.7;
      break;
    case D_RIDICULOUS:
      maxBattery = 12;
      frameRate = 60 - (lasernum * 3.6);
      totalbar = (10.0 - lasernum) * 1.0;
      break;
  }
  frameControl.set(frameRate);
}
