// main.cpp
#include <stdio.h>
#include "gamefunc.h"
#include "cgdlogo.h"
#include "db.h"
#include "meteors.h"
#include "aroids.h"
#include "escfp23x.h"
#include "menu.h"
#include "mapfunc.h"
#include "escpass.h"

enum menutags {M_NOP, M_OPENING, M_GAME, M_DIFFICULTY, M_PLAY, M_INSTRUCTIONS,
  M_SOUND, M_DBSOUND, M_PASSWORD, M_EXIT, M_MAPEDIT,
  M_EASY, M_MEDIUM, M_HARD, M_RIDICULOUS,
  M_DB, M_METEORS, M_AROIDS, M_FINISH};
enum gamereturn {G_SUCCESS, G_AGAIN, G_QUIT};

const WORD CANT = 0;
const WORD CAN_EDB = 1, CAN_EMET = 2, CAN_EAST = 4, CAN_EFIN = 8;
const WORD CAN_MDB = 16, CAN_MMET = 32, CAN_MAST = 64, CAN_MFIN = 128;
const WORD CAN_HDB = 256, CAN_HMET = 512, CAN_HAST = 1024, CAN_HFIN = 2048;
const WORD CAN_RDB = 4096, CAN_RMET = 8192, CAN_RAST = 16384, CAN_RFIN = 32768;
const WORD CAN_E = CAN_EDB | CAN_EMET | CAN_EAST | CAN_EFIN;
const WORD CAN_M = CAN_MDB | CAN_MMET | CAN_MAST | CAN_MFIN;
const WORD CAN_H = CAN_HDB | CAN_HMET | CAN_HAST | CAN_HFIN;
const WORD CAN_R = CAN_RDB | CAN_RMET | CAN_RAST | CAN_RFIN;
const WORD CAN_DB = CAN_EDB | CAN_MDB | CAN_HDB | CAN_RDB;
const WORD CAN_MET = CAN_EMET | CAN_MMET | CAN_HMET | CAN_RMET;
const WORD CAN_AST = CAN_EAST | CAN_MAST | CAN_HAST | CAN_RAST;
const WORD CAN_FIN = CAN_EFIN | CAN_MFIN | CAN_HFIN | CAN_RFIN;

int do_db(void);
int do_meteors(void);
int do_aroids(void);
int do_score(void);
void gameInstructions(void);
void finishInstructions(void);
void do_byebye(void);
void do_endEasy(char *pass);
void do_endMedium(char *pass);
void do_endHard(char *pass);
void do_endRidiculous(void); //char *pass);

class Score
{
public:
  Score(void)
    : dblives(0), msections(0), afuel(0), dbscore(0), mscore(0), ascore(0)
  {}

  void score_db(int l);
  void score_meteors(int s);
  void score_aroids(int f);
  int final(void);

protected:
  void diffSet(void);
  long int score_generic(const char* name, int base, long int mult,
                    long int rem, int inc, int del, int del2);

  int dblives, msections, afuel;
  long int maxScore, gameScore;
  long int dbMult, metMult, aMult;
  long int dbRem, metRem, aRem;
  long int dbscore, mscore, ascore, fscore;
};

const long int TARGETSCORE = 60000;
const long int DBDIV = 3, METDIV = 5, ADIV = 400;

DodgeBeam dodgeBeam;
Asteroids asteroids;

Score score;

  char dodgename[] =           {"DODGEBEAM"};

int main(int argc, char* argv[])
{
  if (argc > 1)
  {
    if (strcmp(argv[1], "-s") == 0)
      speaker.mute();
  }

  sprintf(dodgename,            "DODG%cB%cAM", 240, 240);

  int selection, game, timesIdle = 0;
  int returnCode;
  WORD diffmask, cumdiffmask, gamemask, canI = CAN_EDB;

  textmode(C40);
  _setcursortype(_NOCURSOR);
  randomize();

  Menu mainMenu;
  MenuItem *misound, *midbsound, *mimapedit;
  MenuItem *midifficulty, *migame, *miplay, *miinstructions;

  misound =        new MenuItem("Sound        ", M_SOUND, WHITE);
  midbsound =      new MenuItem("  Sound+     ", M_DBSOUND, WHITE);
  midifficulty =   new MenuItem("Difficulty   ", M_DIFFICULTY, WHITE);
  migame =         new MenuItem("Game         ", M_GAME, WHITE);
  miplay =         new MenuItem("Play         ", M_PLAY, WHITE+BLINK);
  miinstructions = new MenuItem("Instructions ", M_INSTRUCTIONS, WHITE);
  mimapedit =      new MenuItem("Map Edit     ", M_MAPEDIT, WHITE);
  
  mainMenu.setColors(BLUE, BLACK);
  mainMenu.place(14, 7);

  mainMenu.addItem(new MenuItem("Password     ", M_PASSWORD, WHITE));
  mainMenu.addItem(midifficulty);
  mainMenu.addItem(migame);
  mainMenu.addItem(miplay);
  mainMenu.addItem(miinstructions);
  mainMenu.addItem(misound);
  mainMenu.addItem(midbsound);
  mainMenu.addItem(new MenuItem("Exit         ", M_EXIT, RED));
  mainMenu.addItem(mimapedit);

  Menu diffMenu;
  MenuItem *mieasy, *mimedium, *mihard, *miridiculous;
  difficulty = D_EASY;

  mieasy =         new MenuItem("Training     ", M_EASY, GREEN);
  mimedium =       new MenuItem("The Real Deal", M_MEDIUM, BROWN);
  mihard =         new MenuItem("Ewwww...     ", M_HARD, RED);
  miridiculous =   new MenuItem("Bored?       ", M_RIDICULOUS, RED+BLINK);

  diffMenu.setColors(BLUE, BLACK);
  diffMenu.place(14, 18);

  diffMenu.addItem(mieasy);
  diffMenu.addItem(mimedium);
  diffMenu.addItem(mihard);
  diffMenu.addItem(miridiculous);

  Menu gameMenu;
  MenuItem *miopening, *midb, *mimeteors, *miaroids, *mifinish;

  
  miopening =      new MenuItem("Opening/Game ", M_OPENING, WHITE);

  {
    char bob[15];
    sprintf(bob, "%s    ", dodgename);
    midb = new MenuItem(bob, M_DB, LIGHTCYAN);
  }

  mimeteors =      new MenuItem("METE@RS      ", M_METEORS,
    LIGHTMAGENTA, DARKGRAY, 1);
  miaroids =       new MenuItem("ASTER@IDS    ", M_AROIDS,
    BROWN, DARKGRAY, 1);
  mifinish =       new MenuItem("Finish       ", M_FINISH,
    WHITE, DARKGRAY, 0);

  gameMenu.setColors(BLUE, BLACK);
  gameMenu.place(14, 18);

  gameMenu.addItem(miopening);
  gameMenu.addItem(midb);
  gameMenu.addItem(mimeteors);
  gameMenu.addItem(miaroids);
  gameMenu.addItem(mifinish);

  game = M_OPENING;

  PasswordManager pass(19);
 #ifdef CHEAT
  pass.add(new Password(CAN_E | CAN_M | CAN_H | CAN_R, "God"));
 #else
  pass.add(new Password(CAN_E | CAN_M | CAN_H | CAN_R));//, "God"));
 #endif

  pass.add(new Password(CAN_EDB | CAN_EMET));//, "EDB"));
  pass.add(new Password(CAN_EDB | CAN_EMET | CAN_EAST));//, "EMET"));
  pass.add(new Password(CAN_E));//, "EAST"));
  pass.add(new Password(CAN_E | CAN_MDB));//, "EFIN"));

  pass.add(new Password(CAN_E | CAN_MDB | CAN_MMET));//, "MDB"));
  pass.add(new Password(CAN_E | CAN_MDB | CAN_MMET | CAN_MAST));//, "MMET"));
  pass.add(new Password(CAN_E | CAN_M));//, "MAST"));
  pass.add(new Password(CAN_E | CAN_M | CAN_HDB));//, "MFIN"));

  pass.add(new Password(CAN_E | CAN_M | CAN_HDB | CAN_HMET));//, "HDB"));
  pass.add(new Password(CAN_E | CAN_M | CAN_HDB | CAN_HMET | CAN_HAST));//, "HMET"));
  pass.add(new Password(CAN_E | CAN_M | CAN_H));//, "HAST"));
  pass.add(new Password(CAN_E | CAN_M | CAN_H | CAN_RDB));//, "HFIN"));

  pass.add(new Password(CAN_E | CAN_M | CAN_H | CAN_RDB | CAN_RMET));//, "RDB"));
  pass.add(new Password(CAN_E | CAN_M | CAN_H | CAN_RDB | CAN_RMET | CAN_RAST));//, "RMET"));
  pass.add(new Password(CAN_E | CAN_M | CAN_H | CAN_R));//, "RAST"));

  drawlogo();
  opening();

  do
  {
    cset(WHITE);
    clrscr();

    mieasy->changeColorTo(GREEN);
    mimedium->changeColorTo(BROWN);
    mihard->changeColorTo(RED);
    miridiculous->changeColorTo(RED+BLINK);

    mieasy->setHidden(!(canI & CAN_E));
    mimedium->setHidden(!(canI & CAN_M));
    mihard->setHidden(!(canI & CAN_H));
    miridiculous->setHidden(!(canI & CAN_R));

    switch(difficulty)
    {
      case D_EASY:
        diffmask = CAN_E;
        cumdiffmask = CANT;
        mieasy->changeColorTo(LIGHTGREEN);
        midifficulty->lookLike(mieasy);
        break;
      case D_MEDIUM:
        diffmask = CAN_M;
        cumdiffmask = CAN_E;
        mimedium->changeColorTo(YELLOW);
        midifficulty->lookLike(mimedium);
        break;
      case D_HARD:
        diffmask = CAN_H;
        cumdiffmask = CAN_E | CAN_M;
        mihard->changeColorTo(LIGHTRED);
        midifficulty->lookLike(mihard);
        break;
      case D_RIDICULOUS:
        diffmask = CAN_R;
        cumdiffmask = CAN_E | CAN_M | CAN_H;
        miridiculous->changeColorTo(LIGHTRED+BLINK);
        midifficulty->lookLike(miridiculous);
        break;
    }

    midb->setEnabled(canI & diffmask & CAN_DB);
    mimeteors->setEnabled(canI & diffmask & CAN_MET);
    miaroids->setEnabled(canI & diffmask & CAN_AST);
    mifinish->setEnabled(canI & diffmask & CAN_FIN);

    switch(game)
    {
      case M_OPENING:
        gamemask = ~CANT;
	migame->lookLike(miopening);
	break;
      case M_DB:
        gamemask = CAN_DB;
        migame->lookLike(midb);
        break;
      case M_METEORS:
        gamemask = CAN_MET;
        migame->lookLike(mimeteors);
        break;
      case M_AROIDS:
        gamemask = CAN_AST;
        migame->lookLike(miaroids);
        break;
      case M_FINISH:
        gamemask = CAN_FIN;
        migame->lookLike(mifinish);
        break;
    }

    miplay->setEnabled(canI & diffmask & gamemask);
    miinstructions->setEnabled(canI & diffmask & gamemask);

    if (game == M_DB)
    {
      midbsound->setHidden(0);

      if (dodgeBeam.getLaserSound())
        midbsound->changeColorTo(WHITE);
      else
        midbsound->changeColorTo(LIGHTGRAY);
    }
    else
      midbsound->setHidden(1);

    if (speaker.isOn())
      misound->changeColorTo(WHITE);
    else
      misound->changeColorTo(LIGHTGRAY);

    mimapedit->setHidden( !((game == M_AROIDS) && (canI & CAN_R)) );

    mainMenu.display();

    timesIdle = 0;

    do{
      selection = mainMenu.update();
      delay(10);
      if (++timesIdle > 6000)
      {
        if (!drawlogo(1))
          opening();
	timesIdle = 0;
	selection = M_NOP;
      }
    }while(selection == -1);

    switch (selection)
    {
      case M_GAME:
        gameMenu.display();

	timesIdle = 0;

        do {
          selection = gameMenu.update();
	  delay(10);
	  if (++timesIdle > 6000)
	  {
            if (!drawlogo(1))
	      opening();
	    timesIdle = 0;
	    selection = M_NOP;
	  }
        } while (selection == -1);

        game = selection;
        break;
      case M_PLAY:
        if (!(canI & diffmask & gamemask))
          break;

        switch(game)
        {
	  case M_OPENING:
	    if (!drawlogo(1))
	      opening();
	    break;
          case M_DB:
            do {
              returnCode = do_db();
            } while(returnCode == G_AGAIN);

            if (returnCode == G_SUCCESS)
              givecode(dodgename, pass.retrieveByKey(
               cumdiffmask | (diffmask & (CAN_DB | CAN_MET)) )  );
            break;
          case M_METEORS:
            do {
              returnCode = do_meteors();
            } while(returnCode == G_AGAIN);
    
            if (returnCode == G_SUCCESS)
              givecode("METE@RS", pass.retrieveByKey(
               cumdiffmask | (diffmask & (CAN_DB | CAN_MET | CAN_AST)) )  );
            break;
          case M_AROIDS:
            do {
              returnCode = do_aroids();
            } while(returnCode == G_AGAIN);
    
            if (returnCode == G_SUCCESS)
                  givecode("ASTER@IDS", pass.retrieveByKey(
               cumdiffmask | diffmask)  );
            break;
          case M_FINISH:
            if (score.final() == G_SUCCESS)
            {
              if (difficulty != D_EASY)
                grandfinale();
              fuel = 100;
              asteroids.endgameC();
              delay(4000);
	      switch(difficulty)
	      {
		case D_EASY:
		  do_endEasy(pass.retrieveByKey(
		   cumdiffmask | diffmask | CAN_MDB)  );
		  break;
		case D_MEDIUM:
		  do_endMedium(pass.retrieveByKey(
		   cumdiffmask | diffmask | CAN_HDB)  );
		  break;
		case D_HARD:
		  do_endHard(pass.retrieveByKey(
		   cumdiffmask | diffmask | CAN_RDB)  );
		  break;
		case D_RIDICULOUS:
		  do_endRidiculous();
		  break;
	      }
	    }
            //do_byebye();
            break;
        }
        break;
      case M_INSTRUCTIONS:
        if (!(canI & diffmask & gamemask))
          break;

        switch(game)
        {
	  case M_OPENING: gameInstructions(); break;
          case M_DB: dodgeBeam.instructions(); break;
          case M_METEORS: meteors_instructions(); break;
          case M_AROIDS: asteroids.instructions(); break;
	  case M_FINISH: finishInstructions(); break;
        }
        break;
      case M_SOUND:
        if (speaker.isOn())
          speaker.mute();
        else
          speaker.on();
        break;
      case M_DBSOUND:
        dodgeBeam.toggleLaserSound();
        break;
      case M_DIFFICULTY:
        diffMenu.display();

	timesIdle = 0;

        do {
          selection = diffMenu.update();
	  delay(10);
	  if (++timesIdle > 6000)
	  {
            if (!drawlogo(1))
	      opening();
	    timesIdle = 0;
	    selection = M_NOP;
	  }
        } while (selection == -1);

        switch(selection)
        {
          case M_EASY: difficulty = D_EASY; break;
          case M_MEDIUM: difficulty = D_MEDIUM; break;
          case M_HARD: difficulty = D_HARD; break;
          case M_RIDICULOUS: difficulty = D_RIDICULOUS; break;
        }

        break;
      case M_PASSWORD:
        char password[21];
        textbox(9, 9, 30, 11);
        gotoxy(10, 10);
        cset(WHITE);
        getstr(password, sizeof(password));
        canI |= pass.check(password);
        break;
      case M_MAPEDIT:
        makeMap();
        break;
    }
  } while (selection != M_EXIT);

  cset(WHITE + BLINK);
  gotoxy(17, 13);
  cputs("GOODBYE.");
  interruptWait(2000);

  while(getkey())
    ;

  cset(LIGHTGRAY);
  textmode(C80);
  _setcursortype(_NORMALCURSOR);
  return 0;
}

int do_db(void)
{
  int returnValue = G_SUCCESS;
  char inpt;

  switch (dodgeBeam.game())
  {
    case 1:
      returnValue = G_QUIT;
      break;
    case 2:
      dodgeBeam.endSequence();
      returnValue = G_SUCCESS;
      break;
    default:
    case 3: lives = 0; break;
    case 10: break;
  }
  score.score_db(dodgeBeam.getLives());
  return returnValue;
}

int do_meteors(void)
{
  int returnValue = G_SUCCESS;
  char inpt;

  switch (meteors())
  {
    case 1:
      sections = 0;
      returnValue = G_QUIT;
      break;
    case 2: endgameB();
      break;
    default:
    case 3: sections = 0; break;
    case 4: gotoxy(1,17);
      cset(LIGHTGRAY);
      cputs("Sorry, but you need 5 sections\r\n");
      cputs("to see the ending.            \r\n");
      cputs("You escaped to your main ship!");
      getch();
      break;
    case 10: break;
  }
  score.score_meteors(sections);
  return returnValue;
}

int do_aroids(void)
{
  int returnValue = G_SUCCESS;
  char inpt;

  switch (asteroids.start())
  {
    default:
    case 1:
      returnValue = G_QUIT;
      break;
    case 2:
      textbox(2,18,30,20,BLACK,BLUE,1);
      gotoxy(3,19);
      cset(LIGHTGRAY);
      cputs("You escaped the asteroids!!");
      getch();
      if (asteroids.getFuel() <= 0)
      {
        asteroids.endgameC();
        textbox(2,18,30,20,BLACK,BLUE,1);
        gotoxy(3,19);
        cset(LIGHTGRAY);
        cputs("You ran out of fuel!!");
        getch();
	returnValue = G_QUIT;
      }
      break;
  }
  score.score_aroids(asteroids.getFuel());
  return returnValue;
}

void Score::score_db(int l)
{
  diffSet();
  dblives = l;
  dbscore = score_generic(dodgename, dblives, dbMult, dbRem, 1, 500, 100);
}

void Score::score_meteors(int s)
{
  diffSet();
  msections = s;
  mscore = score_generic("METE@RS", msections, metMult, metRem, 1, 500, 100);
}

void Score::score_aroids(int f)
{
  diffSet();
  afuel = f;
  ascore = score_generic("ASTER@IDS", afuel, aMult, aRem, 19, 10, 10);
}

int Score::final(void)
{
  diffSet();

  int a;
  const int BIGINC = (maxScore/1000)*10 + 9;

  fscore = 0;

  //tally score
  cset(LIGHTGRAY);
  clrscr();
  textbox(1,1,40,24);
  gotoxy(3, 4);
  cprintf("DODG%cB%cAM score = %ld.", 240, 240, dbscore);
  drawBar(3, 5, 34, dbscore, maxScore, LIGHTRED, RED, ALOT);
  interruptWait(2000);

  gotoxy(5, 6);
  cprintf("METE@RS score = %ld.", mscore);
  drawBar(3, 7, 34, mscore, maxScore, LIGHTRED, RED, ALOT);
  interruptWait(1000);

  gotoxy(3, 8);
  cprintf("ASTER@IDS score = %ld.", ascore);
  drawBar(3, 9, 34, ascore, maxScore, LIGHTRED, RED, ALOT);
  interruptWait(1000);

  gotoxy(3, 10);
  cprintf("DODG%cB%cAM score =  %ld", 240, 240, dbscore);
  drawBar(30, 10, 10, dbscore, maxScore, LIGHTRED, RED, ALOT);
  gotoxy(5, 11);
  cprintf("METE@RS score = +%ld", mscore);
  drawBar(30, 11, 10, mscore, maxScore, LIGHTRED, RED, ALOT);
  gotoxy(3, 12);
  cprintf("ASTER@IDS score = +%ld", ascore);
  drawBar(30, 12, 10, ascore, maxScore, LIGHTRED, RED, ALOT);
  gotoxy(19, 13);
  dline(8, 196);
  gotoxy(7, 14);
  cputs("FINAL score =  0");
  drawBar(30, 14, 10, fscore, maxScore, LIGHTRED, RED, ALOT);
  gotoxy(11, 15);
  cprintf("you need   %ld", maxScore);
  interruptWait(2000);
  do {
    delay(10);
    if (dbscore > 0)
    {
      if (dbscore >= BIGINC)
        a = BIGINC;
      else
        a = dbscore;
      speaker.play(1000);
      dbscore-=a;
      fscore+=a;
    }
    gotoxy(22, 10);
    cprintf("%ld  ", dbscore);
    drawBar(30, 10, 10, dbscore, maxScore, LIGHTRED, RED, ALOT);
    gotoxy(22, 14);
    cprintf("%ld  ", fscore);
    drawBar(30, 14, 10, fscore, maxScore, LIGHTRED, RED, ALOT);
    delay(10);
    speaker.stop();
  } while (dbscore > 0 && kbhit() == 0);

  fscore += dbscore;
  dbscore = 0;
  gotoxy(22, 10);
  cprintf("%ld  ", dbscore);
  drawBar(30, 10, 10, dbscore, maxScore, LIGHTRED, RED, ALOT);
  gotoxy(22, 14);
  cprintf("%ld  ", fscore);
  drawBar(30, 14, 10, fscore, maxScore, LIGHTRED, RED, ALOT);

  do {
    delay(10);
    if (mscore > 0)
    {
      if (mscore >= BIGINC)
        a = BIGINC;
      else
        a = mscore;
      speaker.play(1000);
      mscore-=a;
      fscore+=a;
    }
    gotoxy(22, 11);
    cprintf("%ld  ", mscore);
    drawBar(30, 11, 10, mscore, maxScore, LIGHTRED, RED, ALOT);
    gotoxy(22, 14);
    cprintf("%ld  ", fscore);
    drawBar(30, 14, 10, fscore, maxScore, LIGHTRED, RED, ALOT);
    delay(10);
    speaker.stop();
  } while (mscore > 0 && kbhit() == 0);

  fscore += mscore;
  mscore = 0;
  gotoxy(22, 11);
  cprintf("%ld  ", mscore);
  drawBar(30, 11, 10, mscore, maxScore, LIGHTRED, RED, ALOT);
  gotoxy(22, 14);
  cprintf("%ld  ", fscore);
  drawBar(30, 14, 10, fscore, maxScore, LIGHTRED, RED, ALOT);

  do {
    delay(10);
    if (ascore > 0)
    {
      if (ascore >= BIGINC)
        a = BIGINC;
      else
        a = ascore;
      speaker.play(1000);
      ascore-=a;
      fscore+=a;
    }
    gotoxy(22, 12);
    cprintf("%ld  ", ascore);
    drawBar(30, 12, 10, ascore, maxScore, LIGHTRED, RED, ALOT);
    gotoxy(22, 14);
    cprintf("%ld  ", fscore);
    drawBar(30, 14, 10, fscore, maxScore, LIGHTRED, RED, ALOT);
    delay(10);
    speaker.stop();
  } while (ascore > 0 && kbhit() == 0);

  fscore += ascore;
  ascore = 0;
  gotoxy(22, 12);
  cprintf("%ld  ", ascore);
  drawBar(30, 12, 10, ascore, maxScore, LIGHTRED, RED, ALOT);
  gotoxy(22, 14);
  cprintf("%ld  ", fscore);
  drawBar(30, 14, 10, fscore, maxScore, LIGHTRED, RED, ALOT);

  while(getkey())
    ;

  interruptWait(6000);

  while(getkey())
    ;

  if (fscore >= maxScore)
  {
    return G_SUCCESS;
  }
  else
  {
    do {
    } while (getkey());
    textbox(1,1,40,24);
    gotoxy(5, 13);
    cset(LIGHTGRAY);
    cprintf("Sorry, you need %ld points", maxScore);
    gotoxy(5, 14);
    cputs("to see the final ending");
    getch();
    return G_QUIT;
  }

}

void Score::diffSet(void)
{
  switch(difficulty)
  {
    case D_EASY:
      maxScore = TARGETSCORE/2;
      break;
    case D_MEDIUM:
      maxScore = TARGETSCORE;
      break;
    case D_HARD:
      maxScore = TARGETSCORE*2;
      break;
    case D_RIDICULOUS:
      maxScore = TARGETSCORE*4;
  }

  gameScore = maxScore/3;

  dbMult = gameScore/DBDIV;
  metMult = gameScore/METDIV;
  aMult = gameScore/ADIV;

  dbRem = gameScore%DBDIV;
  metRem = gameScore%METDIV;
  aRem = gameScore%ADIV;
}

long int Score::score_generic(const char* name, int base, long int mult,
                             long int rem, int inc, int del, int del2)
{
  long int theScore = rem, a;

  cset(LIGHTGRAY);
  clrscr();
  textbox(1,1,40,24);
  gotoxy(2, 4);
  cprintf("%s score = %d x %ld = 0", name, base, mult);
  drawBar(3, 5, 34, theScore, maxScore, LIGHTRED, RED, ALOT);

  delay(100);

  while(getkey())
    ;

  interruptWait(2000);

  do {
    if (!kbhit())
      delay(del);
    if (base > 0)
    {
      if (base >= inc)
        a = inc;
      else
        a = base;
      if (!kbhit())
        speaker.play(1000);
      base -= a;
      theScore += a*mult;
    }
    gotoxy(2, 4);
    cprintf("%s score = %d x %ld = %ld ", name, base, mult, theScore);

    drawBar(3, 5, 34, theScore, maxScore, LIGHTRED, RED, ALOT);

    if (theScore == rem)
      speaker.play(50);

    if (!kbhit())
      delay(del2);
    speaker.stop();
  } while (base > 0);

  theScore += base*mult;

  while(getkey())
    ;

  gotoxy(7,23);
  cputs("Press a key to continue");
  getch();

  return theScore;
}

void gameInstructions(void)
{
  cset(BLUE);
  clrscr();
  titlebar("Escape From Planet 23x.",8,8);
  cset(LIGHTGREEN);
  //     0000000001111111111222222222233333333334
  //     1234567890123456789012345678901234567890
  cputs("Universal keys:  In any of the games, F1");
  cputs("toggles sounds, and ESC will pause and  ");
  cputs("give you the option to quit.  F2 will   ");
  cputs("also toggle the \'sound+\' in DODGBAM.  ");
  cputs("\'sound+\' controls whether the lasers    ");
  cputs("make sound all the time or just when    ");
  cputs("they fire.  (It could get annoying.)    \r\n");

  cputs("To get started, go to \'Opening/Game\',   ");
  cputs("and select that menu item.  It is a sub-");
  cputs("menu, where you can select which game to");
  cputs("play.  Then go down and select          ");
  cputs("\'Instructions\' again to read about that ");
  cputs("game\'s cheesy plot and controls.  Then  ");
  cputs("select \'play\' to get started.           \r\n");

  cputs("Passwords: After beating each game, you ");
  cputs("get a password.  The game doesn't save  ");
  cputs("any information, so this is the only way");
  cputs("you can skip games you have beaten after");
  cputs("exiting, so be sure and write it down   ");
  cputs("before going up and entering it at the  ");
  cputs("\'Password\' option to move on.           ");
  gotoxy(1,25);
  cset(GREEN);
  cputs("       --PRESS A KEY TO CONTINUE--    ");
  getch();
}

void finishInstructions(void)
{
  cset(BLUE);
  clrscr();
  titlebar("Escape From Planet 23x.",8,8);
  cset(LIGHTGREEN);
  //     0000000001111111111222222222233333333334
  //     1234567890123456789012345678901234567890
  gotoxy(1,5);
  cputs("Just go up and hit \'Play\' to tally all  ");
  cputs("of your scores and see if you are cool  ");
  cputs("to see the spiffy ending sequence.      \r\n");
  cputs("Just keep a couple of things in mind:   ");
  cputs("-No scores are saved between sessions,  ");
  cputs("so you only get credit for the most     ");
  cputs("recent of each game you have played     ");
  cputs("since starting the program.\r\n");
  cputs("-Chances are you will need a pretty     ");
  cputs("decent score in each game to get to see ");
  cputs("the ending.\r\n");
  cputs("-Your scores are zeroed in the process  ");
  cputs("of checking them, so do it at your own  ");
  cputs("risk.\r\n");

  gotoxy(1,25);
  cset(GREEN);
  cputs("       --PRESS A KEY TO CONTINUE--    ");
  getch();
}

void do_byebye(void)
{
  clrscr();
  textbox(1,1,40,24);
  cset(RED, LIGHTGRAY);
  gotoxy(2,23);
  cputs("                                      ");
  window(2,2,39,23);
  gotoxy(1,22);
    do {
    } while (getkey());
  //    00000000011111111112222222222333333333
  //    12345678901234567890123456789012345678
  type("               CREDITS:\r\n\r\n");

  type("          everything:\r\n");
  type("               Justin Love\r\n\r\n");

  type("          Well, if you got this\r\n");
  type("     far you obviously enjoyed the\r\n");
  type("     game.  Don't worry, it's\r\n");
  type("     freeware, I wouln't charge\r\n");
  type("     anyone for a hacked up version\r\n");
  type("     of one of my first programs,\r\n");
  type("     but contributions are always\r\n");
  type("     welcome.  Send anything,\r\n");
  type("     including questions and \r\n");
  type("     comments to:\r\n\r\n");

  type("               CGD\r\n");
  type("               807 E. Wilson St.\r\n");
  type("               Batavia IL  60510\r\n");
  do {
  } while (getkey());
  cset(GREEN, LIGHTGRAY);
  type("     You can press a key now.\r\n");
  window(1,1,40,25);
  speaker.play(1000);
  delay(100);
  speaker.stop();
  getch();
}

void do_endEasy(char* pass)
{
  clrscr();
  textbox(1,1,40,24);
  cset(RED, LIGHTGRAY);
  gotoxy(2,23);
  cputs("                                      ");
  window(2,2,39,23);
  gotoxy(1,22);
  while (getkey())
    ;
  //    00000000011111111112222222222333333333
  //    12345678901234567890123456789012345678
  //    -----				 -----
  while (getkey())
    ;
  type("     Well, you beat Training mode.\r\n");
  type("     But before you get too proud\r\n");
  type("     of yourself, remember it's\r\n");
  type("     just training for the next\r\n");
  type("     level, so type in your\r\n");
  type("     password and get started,\r\n");
  type("     'cause if you think you've\r\n");
  type("     seen the REAL ending, you\r\n");
  type("     ain't seen nothin' yet.\r\n");

  type("\r\n");
  type("      Your Password is: ");
  cset(WHITE, LIGHTGRAY);
  type(pass); type("\r\n\r\n");
  cset(GREEN, LIGHTGRAY);
  type("     You can press a key now.\r\n");
  window(1,1,40,25);
  speaker.play(1000);
  delay(100);
  speaker.stop();
  getch();
}

void do_endMedium(char* pass)
{
  clrscr();
  textbox(1,1,40,24);
  cset(RED, LIGHTGRAY);
  gotoxy(2,23);
  cputs("                                      ");
  window(2,2,39,23);
  gotoxy(1,22);
  while (getkey())
    ;
  //    00000000011111111112222222222333333333
  //    12345678901234567890123456789012345678
  //    -----				 -----
  while (getkey())
    ;
  type("     Congratulations, you've just\r\n");
  type("     beat the game at something\r\n");
  type("     apporiximating it's original\r\n");
  type("     difficulty level.  Most of it\r\n");
  type("     was written several years ago\r\n");
  type("     and was one of my first c/c++\r\n");
  type("     programs.  The difficlty\r\n");
  type("     levels were added later to\r\n");
  type("     make the games more accessible\r\n");
  type("     and to provide some extra\r\n");
  type("     challange.  If you get bored\r\n");
  type("     or just want to prove how cool\r\n");
  type("     you are, you can try the next\r\n");
  type("     difficulty level, which should\r\n");
  type("     be a little bit more challanging.\r\n");

  type("\r\n");
  type("       Your Password is: ");
  cset(WHITE, LIGHTGRAY);
  type(pass); type("\r\n\r\n");
  cset(GREEN, LIGHTGRAY);
  type("     You can press a key now.\r\n");
  window(1,1,40,25);
  speaker.play(1000);
  delay(100);
  speaker.stop();
  getch();
}

void do_endHard(char* pass)
{
  clrscr();
  textbox(1,1,40,24);
  cset(RED, LIGHTGRAY);
  gotoxy(2,23);
  cputs("                                      ");
  window(2,2,39,23);
  gotoxy(1,22);
  while (getkey())
    ;
  //    00000000011111111112222222222333333333
  //    12345678901234567890123456789012345678
  //    -----				 -----
  while (getkey())
    ;
  type("     OK bucko, you're obviously\r\n");
  type("     pretty persisant.  Internally\r\n");
  type("     the difficulty levels are known\r\n");
  type("     as Easy, Medium, Hard -- and\r\n");
  type("     just for the heck of it I\r\n");
  type("     created Ridiculous just for\r\n");
  type("     people like you.  It's not\r\n");
  type("     really SUPPOSED to be beaten.\r\n");
  type("     But if that seens like a sour\r\n");
  type("     prize, I also let you play with\r\n");
  type("     the map editor for ASTER@IDS\r\n");
  type("     in easy.  It's pretty cryptic,\r\n");
  type("     but in brief: l loads, s saves,\r\n");
  type("     t toggles radar/scanner, r\r\n");
  type("     refreshes, esc quits, Space sets\r\n");
  type("     the current space to the current\r\n");
  type("     tile, Enter toggles between tile\r\n");
  type("     and place mode, and arrows do:\r\n");
  type("     place mode: move around, tile\r\n");
  type("     mode: u/d:+/-1, r/l:+/-5.\r\n");

  type("\r\n\r\n");
  type("       Your Password is: ");
  cset(WHITE, LIGHTGRAY);
  type(pass); type("\r\n\r\n");
  cset(GREEN, LIGHTGRAY);
  type("     You can press a key now.\r\n");
  window(1,1,40,25);
  speaker.play(1000);
  delay(100);
  speaker.stop();
  getch();
}

void do_endRidiculous(void) //char* pass)
{
  clrscr();
  textbox(1,1,40,24);
  cset(RED, LIGHTGRAY);
  gotoxy(2,23);
  cputs("                                      ");
  window(2,2,39,23);
  gotoxy(1,22);
  while (getkey())
    ;
  //    00000000011111111112222222222333333333
  //    12345678901234567890123456789012345678
  //    -----				 -----
  while (getkey())
    ;
  type("     Well, you've done the\r\n");
  type("     impossible.  My design criteria\r\n");
  type("     for Ridiculous was that I -\r\n");
  type("     who has played the game more\r\n");
  type("     more than all others put\r\n");
  type("     together probably ever will -\r\n");
  type("     got crushed by it.\r\n");
  type("     I can't really offer you any\r\n");
  type("     rewards, but if you get here\r\n");
  type("     I'd like to hear about it, and\r\n");
  type("     maybe put your name up on the\r\n");
  type("     webpage so everboy knows that\r\n");
  type("     you have inhuman reflexes.\r\n");

  type("\r\n\r\n");
  /*  what's the point?
  type("       Your Password is: ");
  cset(WHITE, LIGHTGRAY);
  type(pass); type("\r\n\r\n");
  */

  cset(GREEN, LIGHTGRAY);
  type("     You can press a key now.\r\n");
  window(1,1,40,25);
  speaker.play(1000);
  delay(100);
  speaker.stop();
  getch();
}
