// menu.cpp

#include "menu.h"
#include <stdio.h>
#include <string.h>

MenuItem::~MenuItem(void)
{
  if (menuText != NULL)
    delete[] menuText;
}

void MenuItem::display(int x, int y, int bc)
{
  if (!hidden)
  {
    if (copy)
    {
      copy->display(x, y, bc);
      return;
    }
    else
    {
      gotoxy(x, y);
      if (enabled)
        cset(eForeColor, bc);
      else
        cset(dForeColor, bc);
      cputs(menuText);
    }
  }
}

void MenuItem::setMenuText(char* name)
{
  menuText = new char[strlen(name)+1];
  if (menuText == NULL)
  {
    cputs("Unable to allocate menu text memory\n\r");
    getch();
  }
  else
  {
    strcpy(menuText, name);
  }
}

Menu::Menu(void)
  : ulx(1), uly(1), highlight(LIGHTGRAY), normal(BLACK),
    currentItem(0), maxItem(0)
{
  for (int a = 0;a < 25;a++)
    items[a] = NULL;
}

Menu::~Menu(void)
{
  for (int a = 0;a < 25;a++)
  {
    if (items[a] != NULL)
      delete items[a];
  }
}

void Menu::display(void)
{
  for (int a = 0;a < maxItem;a++)
  {
    if (a == currentItem)
      items[a]->display(ulx, uly+a, highlight);
    else
      items[a]->display(ulx, uly+a, normal);
  }
}

int Menu::update(void)
{
  if (maxItem < 1)
    return -1;

  int delta = 0;

  switch(getkey())
  {
    default: break;
    case ENTER:
    case SPACE: return items[currentItem]->select();
    case UP: delta = -1;
      break;
    case DOWN: delta = 1;
      break;
  }

  if (delta != 0)
  {
    items[currentItem]->display(ulx, uly+currentItem, normal);

    do {
      currentItem += delta;
      if (currentItem < 0)
        currentItem = maxItem - 1;
      if (currentItem >= maxItem)
        currentItem = 0;
    } while (items[currentItem]->isHidden());

    items[currentItem]->display(ulx, uly+currentItem, highlight);
  }
  return -1;
}

void Menu::addItem(MenuItem* what)
{
  if (maxItem < 25)
  {
    items[maxItem++] = what;
  }
}



